#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Assess a Software Project based on Quality Models
#
# Copyright (C) Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Authors:
#   Alvaro del Castillo San Felix <acs@bitergia.com>
#

import argparse
import csv
import json
import logging
import operator
import os

import requests

import django
# settings.configure()
os.environ['DJANGO_SETTINGS_MODULE'] = 'django_prosoul.settings'
django.setup()

from dateutil import parser

import matplotlib.pyplot as plot

from elasticsearch import helpers, Elasticsearch

from prosoul.models import QualityModel
from prosoul.prosoul_utils import find_metric_name_field

THRESHOLDS = ["Very Poor", "Poor", "Fair", "Good", "Very Good"]
HEADERS_JSON = {"Content-Type": "application/json"}
MAX_PROJECTS = 10000  # max number of projects to analyze
HTTPS_CHECK_CERT = False


def get_params():
    parser = argparse.ArgumentParser(usage="usage: prosoul_assess.py [options]",
                                     description="Create a Kibana Dashboard to show a Quality Model")
    parser.add_argument("-e", "--elastic-url", required=True,
                        help="Elasticsearch URL with the metrics")
    parser.add_argument('-g', '--debug', action='store_true')
    parser.add_argument('-i', '--index', required=True, help='Index with the metrics')
    parser.add_argument('-m', '--model', required=True,
                        help='Model to be used to build the Dashboard')
    parser.add_argument('-b', '--backend-metrics-data', default='grimoirelab',
                        help='Backend metrics data to use (grimoirelab, ossmeter, ...)')
    parser.add_argument('--from-date', default='1970-01-01',
                        help='Start date from which to compute the metrics (1970-01-01 by default)')
    parser.add_argument('--plot', action='store_true',
                        help='Show a plot with the metrics values')
    parser.add_argument('--csvfile', required=False,
                        help='Generate a CSV file with the scores of the assessment)')
    parser.add_argument('--attribute', help='Generate only the assessment for an attribute')

    return parser.parse_args()


def compute_metric_per_projects_grimoirelab(es_url, es_index, metric_field, metric_data, from_date=None):
    """
    In the current implementation the metrics supported are just counting metrics with
    an optional filtering defined in metric_params.

    :param es_url: Elasticsearch URL
    :param es_index: Elasticsearch index with the metrics
    :param metric_field: field to select the metrics data
    :param metric_data: data to compute the metric
    :param from_date: date from which to compute the metrics
    :return:
    """

    metric_name = metric_data.implementation
    metric_filter = ""  # filter needed to compute the metric
    metric_agg = ""  # aggregation needed to compute the metric
    agg_id = None  # id for the aggregation

    if metric_data.params:
        # In the params we can have filter or aggs
        params = json.loads(metric_data.params)
        # Build the filter if metric_params is defined
        if 'filter' in params:
            metric_filter = json.dumps(params['filter'])
        elif 'aggs' in params:
            agg_id = list(params['aggs'].keys())[0]
            metric_agg = json.dumps(params['aggs'])
            if metric_agg:
                metric_agg = ', "aggs": ' + metric_agg

    if from_date:
        from_date_iso = from_date.isoformat()
        filter_date = """
        {"range" :
            {
                "grimoire_creation_date" : {
                    "gte" : "%s"
                }
            }
        }
        """ % from_date_iso
        metric_filter = metric_filter + "," + filter_date if metric_filter else filter_date

    if metric_filter:
        # Add a , to join this filter to the rest of filters
        metric_filter = ", " + metric_filter

    # Get the total aggregated value for a metrics in GrimoireLab
    project_metrics = []
    es_query = """
    {
      "size": 0,
      "aggs": {
        "3": {
          "terms": {
            "field": "project",
            "size": %i
          } %s
        }
      },
      "query": {
        "bool": {
          "must": [
            {
              "term": {
                "%s": "%s"
              }
            } %s
          ]
        }
      }
    }

    """ % (MAX_PROJECTS, metric_agg, metric_field, metric_name, metric_filter)

    logging.debug(json.dumps(json.loads(es_query), indent=True))

    res = requests.post(es_url + "/" + es_index + "/_search", data=es_query, verify=HTTPS_CHECK_CERT, headers=HEADERS_JSON)
    res.raise_for_status()

    project_buckets = res.json()["aggregations"]["3"]["buckets"]

    logging.info("Total projects found for %s: %i", metric_name, len(project_buckets))

    for pb in project_buckets:
        if not metric_agg:
            metric_value = pb["doc_count"]
            project_metrics.append({"project": pb['key'], "metric": metric_value})
        else:
            metric_value = pb[agg_id]['value']
            project_metrics.append({"project": pb['key'], "metric": metric_value})

    return project_metrics


def compute_metric_per_project_ossmeter(es_url, es_index, metric_field, metric_data):
    # Get the total aggregated value for a metric in the OSSMeter
    # Elasticsearch index with metrics

    metric_name = metric_data.implementation

    project_metrics = []
    es_query = """
    {
      "size": 0,
      "query": {
        "bool": {
          "must": [
            {
              "term": {
                "%s": "%s"
              }
            }
          ]
        }
      },
      "aggs": {
        "3": {
          "terms": {
            "field": "project.keyword"
          },
          "aggs": {
            "2": {
              "max": {
                "field": "metric_es_value"
              }
            }
          }
        }
      }
    }
    """ % (metric_field, metric_name)

    # logging.debug(json.dumps(json.loads(es_query), indent=True))

    res = requests.post(es_url + "/" + es_index + "/_search", data=es_query, verify=HTTPS_CHECK_CERT, headers=HEADERS_JSON)
    res.raise_for_status()

    project_buckets = res.json()["aggregations"]["3"]["buckets"]
    for pb in project_buckets:
        metric_value = pb["2"]["value"]
        project_metrics.append({"project": pb['key'], "metric": metric_value})

    return project_metrics


def compute_metric_per_project(es_url, es_index, metric_data, backend_metrics_data, from_date=None):
    """ Compute the value of a metric for all projects available """
    metric_name = str(metric_data)
    metric_per_project = None
    metric_field = find_metric_name_field(backend_metrics_data)
    if backend_metrics_data == "ossmeter":
        metric_per_project = compute_metric_per_project_ossmeter(es_url, es_index, metric_field, metric_data)
    elif backend_metrics_data == "grimoirelab":
        metric_per_project = compute_metric_per_projects_grimoirelab(es_url, es_index, metric_field, metric_data, from_date)

    return metric_per_project


def assess_attribute(es_url, es_index, attribute, backend_metrics_data, from_date=None):
    """
    Do the assessment for an attribute in the quality model. If a metric does not have thresholds,
    the score for it is 0.

    :param es_url: Elasticsearch URL
    :param es_index: Index with the metrics data
    :param attribute: name of the attribute from which to compute the metrics
    :param backend_metrics_data: grimoirelab and ossmeter are the backend supported now
    :param from_date: initial date from which to compute the metrics
    :return: a dict with metrics as keys and the projects score per each metric as value
    """

    logging.debug('Doing the assessment for attribute: %s', attribute.name)
    # Collect all metrics that are included in the models
    metrics_with_data = []
    atribute_assessment = {}  # Includes the assessment for each metric per project

    for metric in attribute.metrics.all():
        # We need the metric values and the metric indicators
        if metric.data:
            metrics_with_data.append(metric)
        else:
            logging.debug("Can't find data for %s", metric.name)

    logging.debug("Metrics to be included: %s (%s attribute)", metrics_with_data, attribute.name)

    for metric in metrics_with_data:
        atribute_assessment[str(metric.data)] = {}
        metric_value = compute_metric_per_project(es_url, es_index, metric.data, backend_metrics_data, from_date)
        if metric_value:
            for project_metric in metric_value:
                pname = project_metric['project']
                pmetric = project_metric['metric']
                logging.debug("Project %s metric %s value %i", pname, str(metric.data), pmetric)
                logging.debug("Doing the assesment ...")
                score = 0
                if metric.thresholds:
                    for threshold in metric.thresholds.split(","):
                        if project_metric['metric'] > float(threshold):
                            score += 1
                    threshold = score - 1 if score else 0
                    logging.debug("Score %s for %s: %i (%s)", project_metric['project'],
                                  str(metric.data), score, THRESHOLDS[threshold])
                atribute_assessment[str(metric.data)][pname] = score
        else:
            logging.debug("Can't find value for for %s", metric)

    return atribute_assessment


def goals2projects(assessment):
    """
    Converts an goals assessment dict to a projects assessment dict

    :param assessment: the goal assessment dict
    :return: the project assessment dict
    """

    def check_project_dict(projects, project, goal, attr, metric):
        """ Check that the project dict has all the needed entries"""

        if project not in projects:
            project_dict = {}
        else:
            project_dict = projects[project]

        if goal not in project_dict:
            project_dict[goal] = {}

        if attr not in project_dict[goal]:
            project_dict[goal][attr] = {}

        if metric not in project_dict[goal][attr]:
            project_dict[goal][attr][metric] = {}

        return project_dict

    projects = {}

    for goal in assessment:
        for attr in assessment[goal]:
            for metric in assessment[goal][attr]:
                for project in assessment[goal][attr][metric]:
                    projects[project] = check_project_dict(projects, project, goal, attr, metric)
                    projects[project][goal][attr][metric] = assessment[goal][attr][metric][project]

    return projects


def enrich_assessment(assessment):
    """
    Generate one item with a metric score for a project

    :param assessment: A dict with the results of the assessment
    :return:
    """
    for goal in assessment:
        for attr in assessment[goal]:
            for metric in assessment[goal][attr]:
                for project in assessment[goal][attr][metric]:
                    aitem = {
                        "goal": goal,
                        "attribute": attr,
                        "metric": metric,
                        "project": project,
                        "score_" + metric: assessment[goal][attr][metric][project]
                    }
                    yield aitem


def publish_assessment(es_url, es_index, assessment):
    """
    Publish all the scores for the metrics in assessment

    :param es_url: URL for Elasticsearch
    :param es_index: index in Elasticsearch
    :param assessment: dict with the assessment data
    :return:
    """

    scores_index = es_index + "_scores"

    es_conn = Elasticsearch([es_url], timeout=100, verify_certs=HTTPS_CHECK_CERT)

    scores = []

    # Uploading info to the new ES
    for item in enrich_assessment(assessment):
        score = {
            "_index": scores_index,
            "_type": "item",
            "_source": item
        }
        scores.append(score)

    helpers.bulk(es_conn, scores)
    logging.info("Total scores published in %s: %i", scores_index, len(scores))

    return len(scores)


def assess(es_url, es_index, model_name, backend_metrics_data, only_attribute=None, from_date=None):
    """
    Build the assessment for all projects

    :param es_url: Elasticsearch URL
    :param es_index: Elasticsearch index with the metrics data
    :param model_name: Quality model name
    :param backend_metrics_data: backend to be used for getting the metrics (ossmeter or grimoirelab)
    :param only_attribute: do the assessment only for this attribute
    :param from_date: date since which the metrics must be computed
    :return: a dict with the assessment for all goals and attributes at projects level
    """
    logging.debug('Building the assessment for projects ...')

    assessment = {}  # Includes the assessment for each attribute

    # Check that the model exists
    model_orm = None
    try:
        model_orm = QualityModel.objects.get(name=model_name)
    except QualityModel.DoesNotExist:
        logging.error('Can not find the metrics model %s', model_name)
        RuntimeError('Can not find the metrics model %s' + model_name)

    for goal in model_orm.goals.all():
        assessment[goal.name] = {}
        for attribute in goal.attributes.all():
            if only_attribute and attribute.name != only_attribute:
                continue
            res = assess_attribute(es_url, es_index, attribute, backend_metrics_data, from_date)
            assessment[goal.name][attribute.name] = res

    logging.debug(json.dumps(assessment, indent=True))

    publish_assessment(es_url, es_index, assessment)

    return assessment


def extract_metrics(qm_assessment):
    """
    Extract all metrics from a quality model assessment
    :param qm_assessment: a dict with the quality model assessment
    :return: a list with all the metrics
    """

    metrics = []

    for goal in qm_assessment:
        for attr in qm_assessment[goal]:
            for metric in qm_assessment[goal][attr]:
                metrics.append(qm_assessment[goal][attr][metric])

    return metrics


def build_project_big_number(project):
    """
    Get all metrics for a project and compute the average

    :param project: a dict with a project assessment
    :return: the average of all metrics
    """

    average = None
    metrics = []

    metrics = extract_metrics(project)
    average = sum(metrics) / len(metrics)
    average = round(average, 2)

    return average


def dump_csv(csv_file, projects_data):
    """
    Dump the project metric scores to a CSV file

    :param csv_file: file in which to dump the projects metrics score data
    :param projects_data: dict with the projects metrics score
    :return:
    """

    fieldnames = ['project']
    # Get all score fields: we need to cross all projects to be sure all metrics are found
    for project in projects_data:
        for goal in projects_data[project]:
            for attr in projects_data[project][goal]:
                for metric in projects_data[project][goal][attr]:
                    if metric not in fieldnames:
                        fieldnames.append(metric)

    with open(csv_file, 'w', newline='') as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)

        writer.writeheader()

        for project in projects_data:
            row_dict = {"project": project}
            for goal in projects_data[project]:
                for attr in projects_data[project][goal]:
                    for metric in projects_data[project][goal][attr]:
                        row_dict[metric] = projects_data[project][goal][attr][metric]
            writer.writerow(row_dict)


def build_report(assessment, kind, csv_file=None):
    """

    :param assessment: dict with the goals assessment based on a quality model
    :param kind: kind of report to be built
    :param csv_file: export the metric scores to a CSV file
    :return: a dict with the report per each project
    """

    kinds = ['big_number']
    projects_report = {}

    if kind not in kinds:
        raise RuntimeError("Report kind not supported " + kind)

    if kind == 'big_number':
        # Average of all metrics
        projects_data = goals2projects(assessment)
        if csv_file:
            dump_csv(csv_file, projects_data)

        for project in projects_data:
            projects_report[project] = build_project_big_number(projects_data[project])

    return projects_report


def show_report(report_data, kind, plot_data=False):
    """

    Print in standard output a report based on report_data and kind

    :param report_data: a dict with the report data
    :param kind: kind of report in report_data
    :param plot_data: show a plot with the data
    :return:
    """

    kinds = ['big_number']
    plot_name = "Projects score"
    projects_report = {}

    if kind not in kinds:
        raise RuntimeError("Report kind not supported " + kind)

    sorted_report = sorted(report.items(), key=operator.itemgetter(1), reverse=True)
    for item in sorted_report:
        print(item)
    if plot_data:
        scores = [item[1] for item in sorted_report]
        x = range(0, len(scores))
        plot.plot(x, scores)
        plot.title(plot_name)
        plot.show()


if __name__ == '__main__':

    args = get_params()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
        logging.debug("Debug mode activated")
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')
    logging.getLogger("urllib3").setLevel(logging.WARNING)
    logging.getLogger("requests").setLevel(logging.WARNING)

    from_date = None if not args.from_date else parser.parse(args.from_date)

    assessment = assess(args.elastic_url, args.index, args.model, args.backend_metrics_data, args.attribute, from_date)
    report = build_report(assessment, "big_number", args.csvfile)
    show_report(report, "big_number", args.plot)



from django import forms
from django.core.validators import URLValidator

from . import data
from . import data_editor
from prosoul.prosoul_utils import BACKEND_METRICS_DATA

CROSSMINER_URL = 'https://crossminer.biterg.io'
CROSSMINER_METRICS_INDEX = 'ossmeter'


class VisualizationForm(forms.Form):
    """ A form to collect the params needed to build a quality model visualization """

    # Temporal default values to make easier the config process
    KIBANA_URL = CROSSMINER_URL
    ELASTIC_URL = CROSSMINER_URL + '/data'
    INDEX_DATA = CROSSMINER_METRICS_INDEX
    ATTRIBUTE_TEMPLATE = 'AttributeTemplate'

    def __init__(self, *args, **kwargs):
        kwargs['initial'] = {"es_url": self.ELASTIC_URL,
                             "kibana_url": self.KIBANA_URL,
                             "es_index": self.INDEX_DATA,
                             "attribute_template": self.ATTRIBUTE_TEMPLATE}

        super(VisualizationForm, self).__init__(*args, **kwargs)

        es_attrs = {'class': 'form-control'}
        widget = forms.TextInput(attrs=es_attrs)
        widget_select = forms.Select(attrs=es_attrs)

        qmodels = [('', '')]  # Initial empty choice
        for qmodel in data_editor.QualityModelsData().fetch():
            qmodels += ((qmodel.name, qmodel.name),)

        self.fields['quality_model'] = forms.ChoiceField(label='Quality Model', required=True,
                                                         widget=widget_select, choices=qmodels)

        templates = []
        for template_tuple in data.VizTemplatesData().fetch():
            templates += (template_tuple,)

        self.fields['attribute_template'] = forms.ChoiceField(label='Attribute Template', required=True,
                                                              widget=widget_select, choices=templates)

        backends = []
        for backend in BACKEND_METRICS_DATA:
            backends += ((backend, backend),)

        self.fields['backend_metrics_data'] = forms.ChoiceField(label='Backend for metrics data', required=True,
                                                                widget=widget_select, choices=backends)
        # self.fields['quality_model'].widget = forms.Select()
        self.fields['es_url'] = forms.CharField(label='Elasticsearch URL', max_length=100, widget=widget)
        self.fields['es_url'].validators = [URLValidator()]
        self.fields['kibana_url'] = forms.CharField(label='Kibana URL', max_length=100, widget=widget)
        self.fields['kibana_url'].validators = [URLValidator()]
        self.fields['es_index'] = forms.CharField(label='Index with metrics data', max_length=100, widget=widget)


class AssessmentForm(forms.Form):
    """ A form to collect the params needed to build an assessment of projects """

    # Temporal default values to make easier the config process
    ELASTIC_URL = CROSSMINER_URL + '/data'
    INDEX_DATA = CROSSMINER_METRICS_INDEX

    def __init__(self, *args, **kwargs):
        kwargs['initial'] = {"es_url": self.ELASTIC_URL,
                             "es_index": self.INDEX_DATA}

        super(AssessmentForm, self).__init__(*args, **kwargs)

        es_attrs = {'class': 'form-control'}
        widget = forms.TextInput(attrs=es_attrs)
        widget_select = forms.Select(attrs=es_attrs)

        qmodels = [('', '')]  # Initial empty choice

        for qmodel in data_editor.QualityModelsData().fetch():
            qmodels += ((qmodel.name, qmodel.name),)

        self.fields['quality_model'] = forms.ChoiceField(label='Quality Model', required=True,
                                                         widget=widget_select, choices=qmodels)

        backends = []
        for backend in BACKEND_METRICS_DATA:
            backends += ((backend, backend),)

        self.fields['backend_metrics_data'] = forms.ChoiceField(label='Backend for metrics data', required=True,
                                                                widget=widget_select, choices=backends)
        self.fields['es_url'] = forms.CharField(label='Elasticsearch URL', max_length=100, widget=widget)
        self.fields['es_url'].validators = [URLValidator()]
        self.fields['es_index'] = forms.CharField(label='Index with metrics data', max_length=100, widget=widget)



import functools
import json

from datetime import datetime
from time import time

from django import shortcuts
from django.http import Http404

from django.http import HttpResponse
from django.template import loader

from django.core.files.storage import default_storage
from django.core.files.base import ContentFile

from django.db.utils import IntegrityError

from prosoul.prosoul_export import fetch_models
from prosoul.prosoul_import import convert_to_grimoirelab, feed_models, SUPPORTED_FORMATS
from prosoul.models import Attribute, Goal, Metric, MetricData, QualityModel

from . import forms_editor


#
# Logic not moved inside classes yet
#

def perfdata(func):
    @functools.wraps(func)
    def decorator(*args, **kwargs):
        task_init = time()
        data = func(*args, **kwargs)
        print("%s: %0.3f sec" % (func, time() - task_init))
        return data
    return decorator


@perfdata
def build_forms_context(state=None):
    """ Get all forms to be shown in the editor """
    qmodel_form = forms_editor.QualityModelsForm(state=state)
    add_qmodel_form = forms_editor.QualityModelForm(state=state)
    goals_form = forms_editor.GoalsForm(state=state)
    goal_form = forms_editor.GoalForm(state=state)
    goal_remove_form = forms_editor.GoalForm(state=state)
    attributes_form = forms_editor.AttributesForm(state=state)
    attribute_form = forms_editor.AttributeForm(state=state)
    attribute_remove_form = forms_editor.AttributeForm(state=state)
    metrics_form = forms_editor.MetricsForm(state=state)
    metric_form = forms_editor.MetricForm(state=state)
    metric_data_form = forms_editor.MetricDataForm(state=state)

    if state:
        if state.qmodel_id:
            qmodel_form.initial['id'] = state.qmodel_id
            qmodel_form.initial['name'] = QualityModel.objects.get(id=state.qmodel_id).name
        if state.goals:
            goals_form.initial['id'] = state.goals[0]
            goal_name = Goal.objects.get(id=state.goals[0]).name
            goals_form.initial['name'] = goal_name
            goal_form.initial['goal_name'] = goal_name
            goal_remove_form.initial['goal_name'] = state.goals[0]
        if state.attributes:
            attributes_form.initial['id'] = state.attributes[0]
            attr_name = Attribute.objects.get(id=state.attributes[0]).name
            attributes_form.initial['name'] = attr_name
            attribute_form.initial['attribute_name'] = attr_name
            attribute_remove_form.initial['attribute_name'] = attr_name
        if state.metrics:
            metrics_form.initial['id'] = state.metrics[0]
            metric_name = Metric.objects.get(id=state.metrics[0]).name
            metrics_form.initial['name'] = metric_name
            attribute_form.initial['metric_id'] = state.metrics[0]

    context = {"qmodels_form": qmodel_form,
               "qmodel_form": add_qmodel_form,
               "goals_form": goals_form,
               "goal_form": goal_form,
               "goal_remove_form": goal_remove_form,
               "attributes_form": attributes_form,
               "attribute_form": attribute_form,
               "attribute_remove_form": attribute_remove_form,
               "metrics_form": metrics_form,
               "metric_form": metric_form,
               "metric_data_form": metric_data_form
               }
    context['active_page'] = 'editor'
    return context


@perfdata
def editor(request):
    """ Shows the Prosoul Editor """

    context = build_forms_context()

    return shortcuts.render(request, 'prosoul/editor.html', context)


def import_from_file(request):

    if request.method == "POST":
        myfile = request.FILES["imported_file"]
        cur_dt = datetime.now().strftime("%Y-%m-%d_%H%M%S")
        file_name = "%s_%s.json" % (myfile, cur_dt)
        fpath = '.imported/' + file_name  # FIXME Define path where all these files must be saved
        save_path = default_storage.save(fpath, ContentFile(myfile.read()))

        task_init = time()

        with open(save_path) as fmodel:
            models_json = {}
            import_models_json = json.load(fmodel)

            # Detect the format automatically
            for fmt in SUPPORTED_FORMATS:
                try:
                    models_json = convert_to_grimoirelab(fmt, import_models_json)
                except Exception as ex:
                    print("%s is not in format %s" % (myfile, fmt))
                    continue

                try:
                    feed_models(models_json)
                    break
                except Exception as ex:
                    pass

            if not models_json:
                raise RuntimeError("File %s couldn't be imported." % myfile.name)

        print("Total loading time ... %.2f sec", time() - task_init)

    return shortcuts.redirect("/")


def export_to_file(request, qmodel=None):

    if (request.method == "GET") and (not qmodel):
        return editor(request)

    if request.method == "POST":
        qmodel_id = request.POST["id"]
        qmodel = QualityModel.objects.get(id=qmodel_id)

    file_name = "qmodel_%s.json" % qmodel
    task_init = time()
    try:
        models_json = fetch_models(qmodel)
    except (QualityModel.DoesNotExist, Exception) as ex:
        print(ex)
        error_msg = "Goals from qmodel \"%s\" couldn't be exported." % qmodel
        if request.method == "POST":
            # If request comes from web UI and fails, return error page
            return return_error(error_msg)
        else:
            # If request comes as a GET request, return HTTP 404: Not Found
            return HttpResponse(status=404)

    print("Total loading time ... %.2f sec", time() - task_init)
    if models_json:
        models = json.dumps(models_json, indent=True, sort_keys=True)
        response = HttpResponse(models, content_type="application/json")
        response['Content-Disposition'] = 'attachment; filename=' + file_name
        return response
    else:
        error_msg = "There are no goals to export"
        return return_error(error_msg)

    return editor(request)


def return_error(message):

    template = loader.get_template('prosoul/error.html')
    context = {"alert_message": message}
    render_error = template.render(context)
    return HttpResponse(render_error)

#
# Classes implementing the logic
#


class EditorState():

    def __init__(self, qmodel_id=None, goals=None, attributes=None,
                 metrics=None, form=None):
        self.qmodel_id = qmodel_id
        self.goals = goals
        self.attributes = attributes
        self.metrics = metrics

        if form:
            # The form includes the state not changed to be propagated
            qmodels_state = form.cleaned_data['qmodel_state']
            goals_state = form.cleaned_data['goals_state']
            attributes = form.cleaned_data['attributes_state']
            metrics = form.cleaned_data['metrics_state']

            if self.qmodel_id is None:
                self.qmodel_id = qmodels_state if qmodels_state else None
            if self.goals is None:
                self.goals = [goals_state] if goals_state else []
            if self.attributes is None:
                self.attributes = [attributes] if attributes else []
            if self.metrics is None:
                self.metrics = [metrics] if metrics else []

    def is_empty(self):
        return not (self.qmodel_id or self.goals or self.attributes or self.metrics)

    def initial_state(self):
        """ State to be filled in the forms so it is propagated

        The state needs to be serialized so it can be used in
        forms fields.
        """
        initial = {}

        initial['qmodel_state'] = self.qmodel_id

        if self.goals:
            initial['goals_state'] = ";".join([str(oid) for oid in self.goals])
        else:
            initial['goals_state'] = None

        if self.attributes:
            initial['attributes_state'] = ";".join([str(oid) for oid in self.attributes])
        else:
            initial['attributes_state'] = None

        if self.metrics:
            initial['metrics_state'] = ";".join([str(oid) for oid in self.metrics])
        else:
            initial['metrics_state'] = None

        return initial


class QualityModelView():

    @staticmethod
    def select_qmodel(request, context=None):
        template = "prosoul/editor.html"
        if request.method == 'POST':
            form = forms_editor.QualityModelsForm(request.POST)
            if form.is_valid():
                model_id = form.cleaned_data['id']
                if not model_id:
                    return shortcuts.render(request, template, build_forms_context())
                # Select and qmodel reset the state. Don't pass form=form
                state = build_forms_context(EditorState(qmodel_id=model_id))
                if context:
                    context.update(state)
                else:
                    context = state
                return shortcuts.render(request, template, context)
            else:
                # Ignore when the empty option is selected
                return shortcuts.render(request, template, build_forms_context())
        else:
            return shortcuts.render(request, template, build_forms_context())

    @staticmethod
    def add_qmodel(request):
        if request.method == 'POST':
            form = forms_editor.QualityModelForm(request.POST)
            if form.is_valid():
                qmodel_name = form.cleaned_data['qmodel_name']

                try:
                    qmodel_orm = QualityModel.objects.get(name=qmodel_name)
                except QualityModel.DoesNotExist:
                    qmodel_orm = QualityModel(name=qmodel_name)
                    qmodel_orm.save()

                # Select and qmodel reset the state. Don't pass form=form
                return shortcuts.render(request, 'prosoul/editor.html',
                                        build_forms_context(EditorState(qmodel_id=qmodel_orm.id)))
            else:
                # TODO: Show error
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def remove_qmodel(request):
        errors = None
        if request.method == 'POST':
            form = forms_editor.QualityModelForm(request.POST)
            if form.is_valid():
                qmodel_name = form.cleaned_data['qmodel_name']
                try:
                    QualityModel.objects.get(name=qmodel_name).delete()
                except QualityModel.DoesNotExist:
                    errors = "Can't delete not found quality model %s" % qmodel_name
            else:
                error = form.errors

            # Select and qmodel reset the state. Don't pass form=form
            context = build_forms_context(EditorState(qmodel_id=None))
            context['errors'] = errors

            return shortcuts.render(request, 'prosoul/editor.html', context)

        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def update_qmodel(request):
        errors = None
        if request.method == 'POST':
            form = forms_editor.QualityModelForm(request.POST)
            if form.is_valid():
                qmodel_id = form.cleaned_data['qmodel_state']
                qmodel_name = form.cleaned_data['qmodel_name']
                try:
                    qmodel_orm = QualityModel.objects.get(id=qmodel_id)
                    qmodel_orm.name = qmodel_name
                    qmodel_orm.save()
                except QualityModel.DoesNotExist:
                    errors = "Can't find quality model %s" % qmodel_id
            else:
                error = form.errors

            context = build_forms_context(EditorState(form=form))
            context['errors'] = errors

            return shortcuts.render(request, 'prosoul/editor.html', context)

        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())


class MetricDataView():

    @staticmethod
    def add_metric_data(request):
        if request.method == 'POST':
            form = forms_editor.MetricDataForm(request.POST)
            if form.is_valid():
                state = EditorState(form=form)
                metric_id = state.metrics[0]
                implementation = form.cleaned_data['implementation']
                params = form.cleaned_data['params']

                # Try to find a metric data already created
                try:
                    metric_data_orm = MetricData.objects.get(implementation=implementation, params=params)
                except MetricData.DoesNotExist:
                    # Create a new metric
                    metric_data_orm = MetricData(implementation=implementation, params=params)
                    metric_data_orm.save()

                metric_orm = Metric.objects.get(id=metric_id)
                metric_orm.data = metric_data_orm
                metric_orm.save()

                # form.cleaned_data['metrics_state'] = []
                state = EditorState(form=form)
                return shortcuts.render(request, 'prosoul/editor.html',
                                        build_forms_context(state))
            else:
                # TODO: Show error
                print(form.errors)
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())


class MetricView():

    @staticmethod
    def add_metric(request):
        error = None
        if request.method == 'POST':
            form = forms_editor.MetricForm(request.POST)
            if form.is_valid():
                name = form.cleaned_data['metric_name']
                thresholds = form.cleaned_data['metric_thresholds']
                attribute_id = form.cleaned_data['attributes']

                attribute_orm = Attribute.objects.get(id=attribute_id)

                # Try to find a metric already created
                try:
                    metric_orm = Metric.objects.get(name=name)
                    error = "The metric %s already exists" % name
                    # metric_orm.attribute = attribute_orm
                    # metric_orm.thresholds = thresholds
                except Metric.DoesNotExist:
                    # Create a new metric
                    metric_orm = Metric(name=name, attribute=attribute_orm,
                                        thresholds=thresholds)

                metric_orm.save()

                attribute_orm.metrics.add(metric_orm)
                attribute_orm.save()
            else:
                error = form.errors

            form.cleaned_data['metrics_state'] = []
            state = EditorState(form=form)
            context = build_forms_context(state)
            context.update({"errors": error})

            return shortcuts.render(request, 'prosoul/editor.html',
                                    context)
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def remove_metric(request):
        if request.method == 'POST':
            form = forms_editor.MetricForm(request.POST)
            if form.is_valid():
                if form.cleaned_data['metric_id']:
                    metric_id = int(form.cleaned_data['metric_id'])
                    Metric.objects.get(id=metric_id).delete()
                # Clean from the state the removed metric view
                form.cleaned_data['metrics_state'] = []
                return shortcuts.render(request, 'prosoul/editor.html',
                                        build_forms_context(EditorState(form=form)))
            else:
                # TODO: Show error
                print(form.errors)
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def select_metric(request):
        error = None
        template = 'prosoul/editor.html'
        if request.method == 'POST':
            form = forms_editor.MetricsForm(request.POST)
            if form.is_valid():
                metric_id = int(form.cleaned_data['id'])
                metrics = [metric_id]
                old_metrics = form.cleaned_data['metrics_state']
                if old_metrics == str(metric_id):
                    # Unselect the metric
                    form.cleaned_data['metrics_state'] = None
                    metrics = None

                state = EditorState(form=form, metrics=metrics)
            else:
                state = EditorState(form=form)
                error = form.errors

            context = build_forms_context(state)
            context.update({"errors": error})
            return shortcuts.render(request, template, context)

        else:
            return shortcuts.render(request, template, build_forms_context())

    @staticmethod
    def update_metric(request):
        if request.method == 'POST':
            form = forms_editor.MetricForm(request.POST)

            if form.is_valid():
                metric_id = form.cleaned_data['metric_id']
                name = form.cleaned_data['metric_name']
                attribute_id = form.cleaned_data['attributes']
                metric_data_id = form.cleaned_data['metrics_data']
                old_attribute_id = form.cleaned_data['old_attribute_id']
                thresholds = form.cleaned_data['metric_thresholds']

                metric_orm = Metric.objects.get(id=metric_id)
                metric_orm.name = name
                metric_orm.data = None
                metric_orm.thresholds = thresholds
                if metric_data_id:
                    metric_data_orm = MetricData.objects.get(id=metric_data_id)
                    metric_orm.data = metric_data_orm
                metric_orm.save()

                # Add the metric to the new attribute
                if attribute_id != old_attribute_id:
                    attribute = Attribute.objects.get(id=attribute_id)
                    attribute.metrics.add(metric_orm)
                    attribute.save()
                    attribute = Attribute.objects.get(name=old_attribute_id)
                    attribute.metrics.remove(metric_orm)
                    attribute.save()

                state = EditorState(metrics=[metric_id], form=form)
                return shortcuts.render(request, 'prosoul/editor.html',
                                        build_forms_context(state))
            else:
                # TODO: Show error
                print(form.errors)
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())


class AttributeView():

    @staticmethod
    def add_attribute(request):
        error = None
        if request.method == 'POST':
            form = forms_editor.AttributeForm(request.POST)
            if form.is_valid():
                goal_id = form.cleaned_data['goals_state']
                parent_id = form.cleaned_data['parent_id']
                goal_orm = None

                attribute_name = form.cleaned_data['attribute_name']
                attribute_orm = Attribute(name=attribute_name)
                try:
                    attribute_orm.save()

                    if parent_id:
                        # If there is an attribute parent use it instead of goal
                        parent_orm = Attribute.objects.get(name=parent_id)
                        parent_orm.subattributes.add(attribute_orm)
                        parent_orm.save()
                    elif goal_id:
                        goal_orm = Goal.objects.get(id=goal_id)
                        goal_orm.attributes.add(attribute_orm)
                        goal_orm.save()
                except IntegrityError:
                    attribute_orm = Attribute.objects.get(name=attribute_name)
                    error = "Attribute %s alredy exists" % (attribute_name)

                context = EditorState(attributes=[attribute_orm.id], form=form)
                send_context = build_forms_context(context)
                send_context.update({"errors": error})
                return shortcuts.render(request, 'prosoul/editor.html', send_context)
            else:
                # TODO: Show error
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def select_attribute(request, context=None):
        error = None
        template = 'prosoul/editor.html'
        if request.method == 'POST':
            form = forms_editor.AttributesForm(request.POST)
            if form.is_valid():
                attr_id = form.cleaned_data['id']
                attributes = [attr_id] if attr_id else []
                old_attributes = form.cleaned_data['attributes_state']
                if old_attributes == attr_id:
                    # Unselect the attribute and its metrics
                    form.cleaned_data['attributes_state'] = None
                    form.cleaned_data['metrics_state'] = None
                    attributes = None
                state = EditorState(form=form, attributes=attributes)

                return shortcuts.render(request, template,
                                        build_forms_context(state))
            else:
                state = EditorState(form=form)
                error = form.errors

            if context:
                context.update(build_forms_context(state))
            else:
                context = build_forms_context(state)

            context.update({"errors": error})
            return shortcuts.render(request, template, context)

        else:
            return shortcuts.render(request, template, build_forms_context())

    @staticmethod
    def remove_attribute(request):
        error = None
        if request.method == 'POST':
            form = forms_editor.AttributeForm(request.POST)
            if form.is_valid():
                attribute_name = form.cleaned_data['attribute_name']
                current_id = form.cleaned_data['current_id']
                try:
                    Attribute.objects.get(id=current_id).delete()
                except Attribute.DoesNotExist:
                    error = "Can't remove %s. Attribute does not exists." % attribute_name
                send_context = build_forms_context(EditorState(form=form, attributes=[]))
                send_context.update({"errors": error})
                return shortcuts.render(request, 'prosoul/editor.html', send_context)
            else:
                # TODO: Show error
                print("attribute_goal", form.errors)
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def update_attribute(request):
        if request.method == 'POST':
            form = forms_editor.AttributeForm(request.POST)

            if form.is_valid():
                name = form.cleaned_data['attribute_name']
                current_id = form.cleaned_data['current_id']

                attribute_orm = Attribute.objects.get(id=current_id)
                attribute_orm.name = name
                attribute_orm.save()

                state = EditorState(attributes=[current_id], form=form)
                return shortcuts.render(request, 'prosoul/editor.html',
                                        build_forms_context(state))
            else:
                # TODO: Show error
                print(form.errors)
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())


class GoalView():

    @staticmethod
    def add_goal(request):
        error = None
        if request.method == 'POST':
            form = forms_editor.GoalForm(request.POST)
            if form.is_valid():
                qmodel_id = form.cleaned_data['qmodel_state']
                qmodel_orm = None
                goal_name = form.cleaned_data['goal_name']
                goal_orm = Goal(name=goal_name)
                try:
                    goal_orm.save()
                    if qmodel_id:
                        qmodel_orm = QualityModel.objects.get(id=qmodel_id)
                        qmodel_orm.goals.add(goal_orm)
                        qmodel_orm.save()
                except IntegrityError:
                    error = "Goal %s alredy exists" % (goal_name)
                    goal_orm = Goal.objects.get(name=goal_name)

                context = EditorState(goals=[goal_orm.id], form=form)
                send_context = build_forms_context(context)
                send_context.update({"errors": error})
                return shortcuts.render(request, 'prosoul/editor.html', send_context)
            else:
                # TODO: Show error
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def remove_goal(request):
        error = None
        if request.method == 'POST':
            form = forms_editor.GoalForm(request.POST)
            if form.is_valid():
                goal_name = form.cleaned_data['goal_name']
                current_id = form.cleaned_data['current_id']
                try:
                    Goal.objects.get(id=current_id).delete()
                except Goal.DoesNotExist:
                    error = "Can't remove %s. Goal does not exists." % goal_name
                send_context = build_forms_context(EditorState(goals=[], form=form))
                send_context.update({"errors": error})
                return shortcuts.render(request, 'prosoul/editor.html', send_context)
            else:
                # TODO: Show error
                print("remove_goal", form.errors)
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())

    @staticmethod
    def select_goal(request, context=None):
        error = None
        template = 'prosoul/editor.html'
        if request.method == 'POST':
            form = forms_editor.GoalsForm(request.POST)
            if form.is_valid():
                old_goals = form.cleaned_data['goals_state']
                goal_id = form.cleaned_data['id']
                goals = [goal_id]
                if old_goals == goal_id:
                    # Unselect the goal and its attributes and metrics
                    form.cleaned_data['goals_state'] = None
                    form.cleaned_data['attributes_state'] = None
                    form.cleaned_data['metrics_state'] = None
                    goals = None
                state = EditorState(goals=goals, form=form)
            else:
                state = EditorState(form=form)
                error = form.errors

            if context:
                context.update(build_forms_context(state))
            else:
                context = build_forms_context(state)

            context.update({"errors": error})
            return shortcuts.render(request, template, context)

        else:
            return shortcuts.render(request, template, build_forms_context())

    @staticmethod
    def update_goal(request):
        if request.method == 'POST':
            form = forms_editor.GoalForm(request.POST)

            if form.is_valid():
                goal_name = form.cleaned_data['goal_name']
                current_id = form.cleaned_data['current_id']

                goal_orm = Goal.objects.get(id=current_id)
                goal_orm.name = goal_name
                goal_orm.save()

                state = EditorState(goals=[current_id], form=form)
                return shortcuts.render(request, 'prosoul/editor.html',
                                        build_forms_context(state))
            else:
                # TODO: Show error
                print(form.errors)
                raise Http404
        else:
            return shortcuts.render(request, 'prosoul/editor.html', build_forms_context())



# Generated by Django 2.0.4 on 2018-04-25 04:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('prosoul', '0002_auto_20180307_2051'),
    ]

    operations = [
        migrations.AddField(
            model_name='metricdata',
            name='params',
            field=models.CharField(blank=True, max_length=1024, null=True),
        ),
    ]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Export a Metrics Model to a file
#
# Copyright (C) 2017 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Authors:
#   Alvaro del Castillo San Felix <acs@bitergia.com>
#

import argparse
import json
import logging
import os
import sys

from time import time

import django
# settings.configure()
os.environ['DJANGO_SETTINGS_MODULE'] = 'django_prosoul.settings'
django.setup()

from prosoul.models import QualityModel


def get_params():
    parser = argparse.ArgumentParser(usage="usage: prosoul_export.py [options]",
                                     description="Export a Metrics Model to a file")
    parser.add_argument("-f", "--file", required=True,
                        help="File path in which to export the Metrics Models")
    parser.add_argument('-g', '--debug', action='store_true')
    parser.add_argument('-m', '--model', help='Model to be exported. If not provided all models will be exported.')
    parser.add_argument('--format', default='grimoirelab',
                        help="Import file format (default grimoirelab)")

    return parser.parse_args()


def fetch_model(model_name):
    """ Fetch a data model from Prosoul and convert it to JSON """

    def fetch_attribute(attribute_orm):
        attribute_json = {"name": attribute_orm.name,
                          "description": attribute_orm.description,
                          "metrics": [],
                          "factoids": [],
                          "subattributes": []}

        for metric_orm in attribute_orm.metrics.all():
            data_source_type_name = None
            if metric_orm.data_source_type:
                data_source_type_name = metric_orm.data_source_type.name
            metric_data_implementation = None
            metric_data_params = None
            if metric_orm.data:
                metric_data_implementation = metric_orm.data.implementation
                metric_data_params = metric_orm.data.params

            metric_json = {
                "name": metric_orm.name,
                "description": metric_orm.description,
                "data_implementation": metric_data_implementation,
                "data_params": metric_data_params,
                "data_source_type": data_source_type_name,
                "thresholds": metric_orm.thresholds
            }
            attribute_json['metrics'].append(metric_json)

        for factoid_orm in attribute_orm.factoids.all():
            data_source_type_name = None
            if factoid_orm.data_source_type:
                data_source_type_name = factoid_orm.data_source_type.name

            factoid_json = {
                "name": factoid_orm.name,
                "description": factoid_orm.description,
                "data_source_type": data_source_type_name

            }
            attribute_json['factoids'].append(factoid_json)

        for subattribute_orm in attribute_orm.subattributes.all():
            attribute_json['subattributes'].append(fetch_attribute(subattribute_orm))

        return attribute_json

    def fetch_goal(goal_orm):
        goal_json = {"name": goal_orm.name, "description": goal_orm.description,
                     "attributes": [], "subgoals": []}

        for attribute_orm in goal_orm.attributes.all():
            goal_json['attributes'].append(fetch_attribute(attribute_orm))

        for subgoal_orm in goal_orm.subgoals.all():
            goal_json['subgoals'].append(fetch_goal(subgoal_orm))

        return goal_json

    model_json = {}

    logging.debug("Fetch the model %s", model_name)

    try:
        model_orm = QualityModel.objects.get(name=model_name)
        model_json['name'] = model_orm.name
        model_json['goals'] = []
    except QualityModel.DoesNotExist:
        logging.error('Can not find model %s', model_name)
        raise

    for goal_orm in model_orm.goals.all():

        goal_json = fetch_goal(goal_orm)

        model_json['goals'].append(goal_json)

    return model_json


def fetch_models(model_name=None):
    models_json = {"qualityModels": []}

    if model_name:
        models_json["qualityModels"].append(fetch_model(model_name))
    else:
        models = QualityModel.objects.all()
        for model in models:
            models_json["qualityModels"].append(fetch_model(model.name))

    return models_json


def select_model(gl_models_json, model_name=None):
    """ Select a model from the list of avaialble models by model_name """

    gl_model_json = None

    gl_models_json = gl_models_json['qualityModels']

    if not model_name:
        gl_model_json = gl_models_json[0]
    else:
        for model in gl_models_json:
            if model['name'] == model_name:
                gl_model_json = model
                break
        if not gl_model_json:
            logging.error("Can not find model %s to export", model_name)
            raise RuntimeError("Can not find model %s to export" % model_name)

    return gl_model_json


def gl2viewer(gl_models_json, model_name=None):
    """ Convert a GrimoireLab JSON quality model to Alambic Viewer format """

    def extract_attributes(alambic_children):
        attributes = {}
        for child in alambic_children:
            if child['type'] == 'attribute':
                attributes[child['mnemo']] = {
                    "description": child['description'],
                    "mnemo": child['mnemo'],
                    "name": child['mnemo']
                }
                if 'children' in child:
                    attributes.update(extract_attributes(child['children']))

        return attributes

    def extract_metrics(alambic_children):
        metrics = {}
        for child in alambic_children:
            if child['type'] == 'metric':
                metrics[child['mnemo']] = {
                    "description": child['description'],
                    "mnemo": child['mnemo'],
                    "name": child['mnemo']
                }
            if 'children' in child and child['children']:
                metrics.update(extract_metrics(child['children']))

        return metrics

    viewer_json = {}
    alambic_json = gl2alambic(gl_models_json, model_name=model_name, viewer=True)

    attributes_json = {"children": extract_attributes(alambic_json['children'])}
    metrics_json = {"children": extract_metrics(alambic_json['children'])}

    viewer_json = [alambic_json]
    viewer_json.append(attributes_json)
    viewer_json.append(metrics_json)

    return viewer_json


def gl2alambic(gl_models_json, model_name=None, viewer=False):
    """
    Convert a GrimoireLab JSON quality model to Alambic format.
    Alambic format does not include name an description fields in its items,
    but these fields are needed for the Alambic web quality model viewer,
    so they are added if the model is going to be used in the quality model viewer.
    A bit hacky, but there is not a formal definition of the format, and in
    the quality model used as reference for Alambic, name and description
    do not exists.

    :param gl_models_json: Models in GrimoireLab format
    :param model_name: model to be converted (alambic only supports one)
    :param viewer: add extra fields needed by Alambic viewer
    :return: a dict with the model in alambic format
    """

    def attribute2child(attribute, viewer=False):
        """ Convert an Alambic child to an attribute """
        al_attribute = {"active": "true", "type": "attribute",
                        "mnemo": attribute['name'],
                        "children": []}
        if viewer:
            al_attribute["description"] = attribute['description']
            al_attribute["name"] = attribute['name']

        for metric in attribute['metrics']:
            al_metric = {"active": "true", "type": "metric",
                         "mnemo": metric['name']}
            if viewer:
                al_metric["description"] = metric['description']
                al_metric["name"] = metric['name']

            al_attribute['children'].append(al_metric)

        if 'subattributes' in attribute:
            for subattribute in attribute['subattributes']:
                al_attribute['children'].append(attribute2child(subattribute, viewer))

        return al_attribute

    def goal2atributte(goal, viewer=False):
        """ In Alambic goals are first level attributes """

        goal_attribute = {"active": "true", "type": "attribute",
                          "mnemo": goal['name'], "children": []}
        if viewer:
            goal_attribute["description"] = goal['description']
            goal_attribute["name"] = goal['name']

        for attribute in goal['attributes']:
            goal_attribute["children"].append(attribute2child(attribute, viewer))

        if 'subgoals' in goal:
            for subgoal in goal['subgoals']:
                goal_attribute["children"].append(goal2atributte(subgoal, viewer))

        return goal_attribute

    alambic_json = {"name": "", "version": "", "children": []}
    gl_model_json = select_model(gl_models_json, model_name)

    alambic_json['name'] = gl_model_json['name']
    alambic_json['version'] = '0.1'  # Version exported

    for goal in gl_model_json['goals']:
        alambic_child = goal2atributte(goal, viewer)
        alambic_json['children'].append(alambic_child)

    return alambic_json


def gl2ossmeter(gl_models_json, model_name=None):
    """ Convert a GrimoireLab JSON quality model to OSSMeter format """

    def goal2qa(goal):
        qaspect = {"qualityAspects": [], "name": goal['name'], "attributes": []}

        for attribute in goal['attributes']:
            qa_attribute = {"name": attribute['name'],
                            "description": attribute['description'],
                            "metrics": [],
                            "factoids": []
                            }
            for metric in attribute['metrics']:
                qa_attribute['metrics'].append(metric['name'])
            for factoid in attribute['factoids']:
                qa_attribute['factoids'].append(factoid['name'])

            qaspect["attributes"].append(qa_attribute)

        if 'subgoals' in goal:
            for subgoal in goal['subgoals']:
                subqaspect = goal2qa(subgoal)
                qaspect['qualityAspects'].append(subqaspect)

        return qaspect

    ossmodel_json = {"qualityModel": {"qualityAspects": []}}
    gl_model_json = select_model(gl_models_json, model_name)

    ossmodel_json["qualityModel"]['name'] = gl_model_json['name']

    for goal in gl_model_json['goals']:
        qualityAspect = goal2qa(goal)
        ossmodel_json['qualityModel']['qualityAspects'].append(qualityAspect)

    return ossmodel_json


def show_report(models_json):

    def report_attribute(attribute, nattributes, nmetrics, nfactoids):
        nmetrics += len(attribute['metrics'])
        if 'factoids' in attribute:
            nfactoids += len(attribute['factoids'])

        if 'subattributes' in attribute:
            for subattribute in attribute['subattributes']:
                nattributes += 1
                (nattributes, nmetrics, nfactoids) = \
                    report_attribute(subattribute, nattributes, nmetrics, nfactoids)
        return (nattributes, nmetrics, nfactoids)

    def report_goal(goal, nattributes, nmetrics, nfactoids, ngoals):
        for attribute in goal['attributes']:
            nattributes += 1
            (nattributes, nmetrics, nfactoids) = \
                report_attribute(attribute, nattributes, nmetrics, nfactoids)

        if 'subgoals' in goal:
            for subgoal in goal['subgoals']:
                ngoals += 1
                (nattributes, nmetrics, nfactoids, ngoals) = \
                    report_goal(subgoal, nattributes, nmetrics, nfactoids, ngoals)

        return (nattributes, nmetrics, nfactoids, ngoals)

    nmodels = 0
    ngoals = 0
    nattributes = 0
    nmetrics = 0
    nfactoids = 0

    for model in models_json['qualityModels']:
        nmodels += 1
        for goal in model['goals']:
            ngoals += 1
            (nattributes, nmetrics, nfactoids, ngoals) = \
                report_goal(goal, nattributes, nmetrics, nfactoids, ngoals)

    print("Models:", nmodels)
    print("Goals:", ngoals)
    print("Attributes:", nattributes)
    print("Metrics:", nmetrics)
    print("Factoids:", nfactoids)


if __name__ == '__main__':

    task_init = time()

    args = get_params()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
        logging.debug("Debug mode activated")
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')
    logging.getLogger("urllib3").setLevel(logging.WARNING)
    logging.getLogger("requests").setLevel(logging.WARNING)

    if os.path.isfile(args.file):
        logging.info("%s exists. Remove it before running.", args.file)
        sys.exit(0)

    with open(args.file, "w") as fmodel:
        try:
            if args.model:
                logging.info("Exporting model %s to file %s", args.model, args.file)
                models_json = fetch_models(args.model)
            else:
                logging.info("Exporting all models to file %s", args.file)
                models_json = fetch_models()

            show_report(models_json)

            if args.format == 'ossmeter':
                models_json = gl2ossmeter(models_json)
            elif args.format == 'alambic':
                models_json = gl2alambic(models_json)
            elif args.format == 'viewer':
                models_json = gl2viewer(models_json)
            elif args.format == 'grimoirelab':
                models_json = models_json
            else:
                raise RuntimeError('Export format not supported ' + args.format)

            if args.format != 'viewer':
                json.dump(models_json, fmodel, indent=True)
            else:
                json.dump(models_json[0], fmodel, indent=True)
                logging.info('Generating extra files for attributes and metrics')
                # attributes_full.json
                attributes_file = os.path.join(os.path.dirname(fmodel.name), "attributes_full.json")
                with open(attributes_file, "w") as fattrs:
                    json.dump(models_json[1], fattrs, indent=True)
                # metrics_full.json
                metrics_file = os.path.join(os.path.dirname(fmodel.name), "metrics_full.json")
                with open(metrics_file, "w") as fmetrics:
                    json.dump(models_json[2], fmetrics, indent=True)

        except Exception:
            os.remove(args.file)
            raise

    logging.debug("Total exporting time ... %.2f sec", time() - task_init)



from django.conf.urls import url
from django.views.generic import RedirectView


from . import views
from . import views_editor


urlpatterns_edit = [
    url(r'^editor$', views_editor.editor, name='editor'),
    url(r'^import$', views_editor.import_from_file),
    url(r'^export/qmodel=(?P<qmodel>[\w ]+)', views_editor.export_to_file),
    url(r'^export$', views_editor.export_to_file),
    url(r'^add_qmodel$', views_editor.QualityModelView.add_qmodel),
    url(r'^remove_qmodel$', views_editor.QualityModelView.remove_qmodel),
    url(r'^select_qmodel$', views_editor.QualityModelView.select_qmodel),
    url(r'^update_qmodel$', views_editor.QualityModelView.update_qmodel),
    url(r'^add_goal$', views_editor.GoalView.add_goal),
    url(r'^remove_goal$', views_editor.GoalView.remove_goal),
    url(r'^select_goal$', views_editor.GoalView.select_goal),
    url(r'^update_goal$', views_editor.GoalView.update_goal),
    url(r'^add_attribute$', views_editor.AttributeView.add_attribute),
    url(r'^remove_attribute$', views_editor.AttributeView.remove_attribute),
    url(r'^select_attribute$', views_editor.AttributeView.select_attribute),
    url(r'^update_attribute$', views_editor.AttributeView.update_attribute),
    url(r'^add_metric$', views_editor.MetricView.add_metric),
    url(r'^remove_metric$', views_editor.MetricView.remove_metric),
    url(r'^select_metric$', views_editor.MetricView.select_metric),
    url(r'^update_metric$', views_editor.MetricView.update_metric),
    url(r'^add_metric_data$', views_editor.MetricDataView.add_metric_data)
]

urlpatterns = [
    url(r'^$', RedirectView.as_view(url='viewer')),
    url(r'^viewer$', views.Viewer.viewer, name='viewer'),
    url(r'^visualize$', views.Visualize.visualize),
    url(r'^create_visualization$', views.Visualize.create),
    url(r'^assess$', views.Assessment.assess),
    url(r'^create_assessment$', views.Assessment.create)
]

urlpatterns += urlpatterns_edit



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-03-07 20:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('prosoul', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='name',
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name='goal',
            name='name',
            field=models.CharField(max_length=200),
        ),
    ]



import functools

from time import time

from django import forms

from prosoul.models import Attribute, Metric, QualityModel

from . import data_editor

SELECT_LINES = 20
MAX_ITEMS = 1000  # Implement pagination if there are more items


def perfdata(func):
    @functools.wraps(func)
    def decorator(self, *args, **kwargs):
        task_init = time()
        data = func(self, *args, **kwargs)
        print("%s: Total data collecting time ... %0.3f sec" %
              (self.__class__.__name__, time() - task_init))
        return data
    return decorator


class ProsoulEditorForm(forms.Form):

    select_widget = forms.Select(attrs={'size': SELECT_LINES, 'class': 'form-control'})
    attrs = {'size': SELECT_LINES, 'class': 'form-control', 'onclick': 'this.form.submit()'}
    select_widget_onclick = forms.Select(attrs=attrs)

    # Hidden widgets to store the state of the ProsoulEditorForm
    qmodel_state = forms.CharField(required=False, max_length=50, widget=forms.HiddenInput())
    goals_state = forms.CharField(required=False, max_length=50, widget=forms.HiddenInput())
    attributes_state = forms.CharField(required=False, max_length=50, widget=forms.HiddenInput())
    metrics_state = forms.CharField(required=False, max_length=50, widget=forms.HiddenInput())

    def is_empty_state(self):
        return self.state.is_empty() if self.state else True

    def __init__(self, *args, **kwargs):
        self.state = kwargs.pop('state') if 'state' in kwargs else None
        if self.state:
            if 'initial' in kwargs:
                kwargs['initial'].update(self.state.initial_state())
            else:
                kwargs['initial'] = self.state.initial_state()
        super(ProsoulEditorForm, self).__init__(*args, **kwargs)

        # The state includes the names of objects except for metrics
        # in which ids are included because there is no name
        self.state_fields = [self['qmodel_state'],
                             self['goals_state'],
                             self['attributes_state'],
                             self['metrics_state']
                             ]


class QualityModelForm(ProsoulEditorForm):

    @perfdata
    def __init__(self, *args, **kwargs):
        super(QualityModelForm, self).__init__(*args, **kwargs)

        qmodel_name = None
        if self.state and self.state.qmodel_id:
            qmodel_id = self.state.qmodel_id
            qmodel_name = QualityModel.objects.get(id=qmodel_id).name

        qm_attrs = {'class': 'form-control', 'placeholder': 'QualityModel name'}
        self.fields['qmodel_name'] = forms.CharField(label='QualityModel name',
                                                     max_length=100, initial=qmodel_name)
        self.fields['qmodel_name'].widget = forms.TextInput(attrs=qm_attrs)


class QualityModelsForm(ProsoulEditorForm):

    select_widget = forms.Select(attrs={'class': 'form-control', 'onclick': 'this.form.submit()'})

    @perfdata
    def __init__(self, *args, **kwargs):
        print(args, kwargs)

        super(QualityModelsForm, self).__init__(*args, **kwargs)

        choices = [('', '')]  # Initial empty choice

        for qmodel in data_editor.QualityModelsData(self.state).fetch():
            choices += ((qmodel.id, qmodel.name),)

        self.fields['id'] = forms.ChoiceField(label='QualityModels', required=True,
                                              widget=self.select_widget, choices=choices)


class GoalForm(ProsoulEditorForm):

    @perfdata
    def __init__(self, *args, **kwargs):
        super(GoalForm, self).__init__(*args, **kwargs)

        self.fields['goal_name'] = forms.CharField(label='Goal name', max_length=100)
        self.fields['goal_name'].widget = forms.TextInput(attrs={'class': 'form-control'})

        current_id = None
        if self.state and self.state.goals:
            current_id = self.state.goals[0]
        self.fields['current_id'] = forms.CharField(required=False, max_length=50,
                                                    widget=forms.HiddenInput(),
                                                    initial=current_id)


class GoalsForm(ProsoulEditorForm):

    @perfdata
    def __init__(self, *args, **kwargs):
        super(GoalsForm, self).__init__(*args, **kwargs)

        choices = ()

        for goal in data_editor.GoalsData(self.state).fetch():
            if (goal.id, goal.name) not in choices:
                choices += ((goal.id, goal.name),)

        choices = sorted(choices, key=lambda x: x[1])
        self.fields['id'] = forms.ChoiceField(label='Goals',
                                              widget=self.select_widget_onclick, choices=choices)


class AttributeForm(ProsoulEditorForm):

    @perfdata
    def __init__(self, *args, **kwargs):
        super(AttributeForm, self).__init__(*args, **kwargs)

        ds_attrs = {'class': 'form-control', 'placeholder': 'Attribute'}
        self.fields['attribute_name'] = forms.CharField(label='Attribute name', max_length=100)
        self.fields['attribute_name'].widget = forms.TextInput(attrs=ds_attrs)

        current_id = None
        if self.state and self.state.attributes:
            current_id = self.state.attributes[0]
        self.fields['current_id'] = forms.CharField(required=False, max_length=50,
                                                    widget=forms.HiddenInput(),
                                                    initial=current_id)
        choices = (('', ''),)

        widget = forms.Select(attrs={'class': 'form-control'})

        for attribute in data_editor.AttributesData(self.state).fetch():
            if (attribute.id, attribute.name) not in choices and \
               attribute.id != current_id:
                choices += ((attribute.id, attribute.name),)

        choices = sorted(choices, key=lambda x: x[1])

        self.fields['parent_id'] = forms.ChoiceField(label='Parent', required=False,
                                                     widget=widget, choices=choices)


class AttributesForm(ProsoulEditorForm):

    def list_choices(self):
        choices = ()

        for attribute in data_editor.AttributesData(self.state).fetch():
            if (attribute.id, attribute.name) not in choices:
                choices += ((attribute.id, attribute.name),)

        choices = sorted(choices, key=lambda x: x[1])

        return choices

    @perfdata
    def __init__(self, *args, **kwargs):
        super(AttributesForm, self).__init__(*args, **kwargs)

        self.fields['id'] = forms.ChoiceField(label='Attributes',
                                              widget=self.select_widget_onclick, choices=self.list_choices())


class MetricDataForm(ProsoulEditorForm):
    def __init__(self, *args, **kwargs):
        super(MetricDataForm, self).__init__(*args, **kwargs)

        ds_attrs = {'class': 'form-control', 'placeholder': 'Implementation'}
        self.fields['implementation'] = forms.CharField(label='Implementation', max_length=100)
        self.fields['implementation'].widget = forms.TextInput(attrs=ds_attrs)
        self.fields['params'] = forms.CharField(label='Params', required=False, max_length=100)
        params_sample = '{"filter": {"term": {"state": "closed"}}}'
        self.fields['params'].widget = forms.TextInput(attrs={'class': 'form-control', 'placeholder': params_sample})


class MetricsForm(ProsoulEditorForm):

    @perfdata
    def __init__(self, *args, **kwargs):
        super(MetricsForm, self).__init__(*args, **kwargs)

        choices = ()

        for metric in data_editor.MetricsData(self.state).fetch():
            choices += ((metric.id, metric),)
            if len(choices) > MAX_ITEMS:
                break

        self.fields['id'] = forms.ChoiceField(label='Metric',
                                              widget=self.select_widget_onclick, choices=choices)


class MetricForm(ProsoulEditorForm):
    # This implementation needs to be revisited and simplified

    @perfdata
    def __init__(self, *args, **kwargs):
        self.metric_id = None
        self.metric_orm = None

        # First state process in order to fill initial values
        kwargs['initial'] = {}
        self.state = kwargs.get('state') if 'state' in kwargs else None
        if self.state:
            kwargs['initial'] = self.state.initial_state()

        if self.state and self.state.metrics:
            self.metric_id = self.state.metrics[0]

        if self.state and self.state.attributes:
            attribute_orm = Attribute.objects.get(id=self.state.attributes[0])
            kwargs['initial'].update({
                'attributes': attribute_orm.id,
                'old_attribute_id': attribute_orm.id
            })

        if self.metric_id:
            try:
                metric_orm = Metric.objects.get(id=self.metric_id)
                kwargs['initial'].update({
                    'metric_id': self.metric_id,
                    'metric_name': metric_orm.name,
                    'metric_thresholds': metric_orm.thresholds
                })
                if metric_orm.data:
                    kwargs['initial'].update({'metrics_data': metric_orm.data.id})
            except Metric.DoesNotExist:
                print(self.__class__, "Received metric which does not exists", self.metric_id)
        super(MetricForm, self).__init__(*args, **kwargs)

        self.fields['metric_id'] = forms.CharField(label='metric_id', required=False, max_length=100)
        self.fields['metric_id'].widget = forms.HiddenInput()

        self.fields['metric_name'] = forms.CharField(label='name', max_length=100, required=False)
        self.fields['metric_name'].widget = forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'metric name'})

        self.fields['metric_thresholds'] = forms.CharField(label='Thresholds', max_length=100, required=False)
        self.fields['metric_thresholds'].widget = forms.TextInput(attrs={'class': 'form-control', 'placeholder': '1,2,3,4,5'})

        choices = ()

        # Show only the attributes for this quality model
        for attr in data_editor.AttributesData(state=self.state).fetch():
            choices += ((attr.id, attr.name),)

        empty_choice = [('', '')]
        choices = empty_choice + sorted(choices, key=lambda x: x[1])

        self.widget = forms.Select(attrs={'class': 'form-control'})
        self.fields['attributes'] = forms.ChoiceField(label='Attributes', required=True,
                                                      widget=self.widget, choices=choices)

        choices = ()

        for metric_data in data_editor.MetricsDataData(state=None).fetch():
            choices += ((metric_data.id, str(metric_data)),)

        empty_choice = [('', '')]
        choices = empty_choice + sorted(choices, key=lambda x: x[1])

        self.fields['metrics_data'] = forms.ChoiceField(label='Metrics Data', required=False,
                                                        widget=self.widget, choices=choices)

        self.fields['old_attribute_id'] = forms.CharField(label='old_attribute', max_length=100, required=False)
        self.fields['old_attribute_id'].widget = forms.HiddenInput(attrs={'class': 'form-control', 'readonly': 'True'})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Create a Kibana Dashboard to show a Quality Model
#
# Copyright (C) Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Authors:
#   Alvaro del Castillo San Felix <acs@bitergia.com>
#

import argparse
import json
import logging
import os

import requests

import django
# settings.configure()
os.environ['DJANGO_SETTINGS_MODULE'] = 'django_prosoul.settings'
django.setup()

from prosoul.data import VizTemplatesData
from prosoul.models import QualityModel
from prosoul.prosoul_assess import assess
from prosoul.prosoul_utils import find_metric_name_field

from kidash.kidash import feed_dashboard

ES_HEADERS = {"Content-Type": "application/json", "kbn-xsrf": "true"}
ASSESS_PANEL = 'panels/scava-projects-radar.json'


def get_params():
    parser = argparse.ArgumentParser(usage="usage: prosoul_vis.py [options]",
                                     description="Create a Kibana Dashboard to show a Quality Model")
    parser.add_argument("-e", "--elastic-url", default="http://localhost:9200",
                        help="Elasticsearch URL with the metrics (http://localhost:9200 by default)")
    parser.add_argument("-k", "--kibana-url", default="http://localhost:5601",
                        help="Kibana URL (http://localhost:9200 by default)")
    parser.add_argument('-g', '--debug', action='store_true')
    parser.add_argument('-i', '--index', required='True', help='Index with the metrics')
    parser.add_argument('-t', '--template-file', required=True,
                        help='Dashboard template file to be used')
    parser.add_argument('--template-assess-file', default=ASSESS_PANEL,
                        help='Dashboard template file to be used for assessment')
    parser.add_argument('-m', '--model', required='True',
                        help='Model to be used to build the Dashboard')
    parser.add_argument('-b', '--backend-metrics-data', default='grimoirelab',
                        help='Backend metrics data to use (grimoirelab, ossmeter, ...)')

    return parser.parse_args()


def build_filters(metrics, metric_name):
    # Basic filter template to be updated
    template = """
    [{"$state": {"store": "appState"},
      "meta": {
        "alias": null,
        "disabled": false,
        "index": "ossmeter",
        "key": "%s",
        "negate": false,
        "params": ["numberOfDocuments", "numberOfBugs"],
        "type": "phrases",
        "value": "numberOfDocuments, numberOfBugs"},
        "query": {"bool": {"minimum_should_match": 1,
                  "should": [{"match_phrase": {"%s": "metric1"}},
                             {"match_phrase": {"%s": "metric2"}}]}}},
      {"query": {"match_all": {}}}]
      """ % (metric_name, metric_name, metric_name)

    filter_json = json.loads(template)

    filter_json[0]['meta']['params'] = metrics
    filter_json[0]['meta']['value'] = ", ".join(metrics)
    filter_should = []
    for metric in metrics:
        filter_should.append({'match_phrase': {metric_name: metric}})
    filter_json[0]['query']['bool']['should'] = filter_should

    return filter_json


def create_alias(es_url, es_index, es_alias):
    """
    Create an alias in Elasticsearch

    :param es_url: Elasticsearch URL
    :param es_index: name of the index
    :param es_alias: name of the alias to be created
    :return:
    """

    add_alias = """{
        "actions": [
            {
                "add": {
                    "index": "%s",
                    "alias": "%s"
                }
            }
        ]
    }""" % (es_index, es_alias)

    res = requests.post(es_url + "/_aliases",
                        headers=ES_HEADERS,
                        data=add_alias, verify=False)
    try:
        res.raise_for_status()
        logging.debug("Created alias %s for index %s" % (es_alias, es_index))
    except requests.exceptions.HTTPError:
        logging.error("Can not create the alias %s for index %s" % (es_alias, es_index))
        raise


def build_dashboard(es_url, kibana_url, es_index, template_filename, goal, attribute,
                    backend_metrics_data):
    """
    Build a Kibana dashboard using as template template_filename showing the data for
    an attribute in a goal from the quality model.

    :param es_url: Elasticsearch URL
    :param kibana_url: Kibana URL
    :param es_index: index with the metrics data
    :param template_filename: template to be used to create the dashboard
    :param goal: quality model goal to be included
    :param attribute: atribute in the goal to be included
    :param backend_metrics_data: metrics backend to be used for getting the data
    :return: a dict with the dashboard created
    """

    logging.debug('Building the dashboard for the attribute: %s (goal %s)', attribute, goal)

    # Check that the model and the template dashboard exists

    # Collect metrics to be included in this attribute
    metrics_data = []

    for metric in attribute.metrics.all():
        if metric.data:
            metrics_data.append(metric.data.implementation)
        else:
            logging.warning("The metric %s does not have data", metric.name)

    logging.debug("Metrics to be included: %s", metrics_data)

    # Upload the dashboard created from the template dashboard
    logging.debug("Uploading the template panel %s", template_filename)
    dashboard = VizTemplatesData.read_template(template_filename)

    # An alias is created from the index with the metrics to the template index
    index_pattern_name = dashboard['index_patterns'][0]['id']
    create_alias(es_url, es_index, index_pattern_name)

    # Add the filters to the template dashboard and export it to Kibana
    search_json = json.loads(dashboard['dashboard']['value']['kibanaSavedObjectMeta']['searchSourceJSON'])
    metric_name = find_metric_name_field(backend_metrics_data)
    search_json['filter'] = build_filters(metrics_data, metric_name)
    dashboard['dashboard']['value']['kibanaSavedObjectMeta']['searchSourceJSON'] = json.dumps(search_json)
    dashboard['dashboard']['value']['title'] = goal.name + "_" + attribute.name
    dashboard['dashboard']['id'] = goal.name + "_" + attribute.name

    feed_dashboard(dashboard, es_url, kibana_url)

    logging.info('Created the attribute dashboard %s', dashboard['dashboard']['value']['title'])

    return dashboard


def build_menu(es_url, qm_menu, assessment_menu):
    """
    Build the menu to access the quality model dashboards

    :param es_url: Elasticsearch URL
    :param qm_menu: dict with the quality model menu to be created
    :param assessment_menu: dict with the assessment menu
    :return:
    """

    logging.debug("Creating the menu for accessing %s" % qm_menu)

    # {'Product_Vitality': 'Product_Vitality', 'Community_Attention': 'Community_Attention'}
    #
    # must be converted to
    #
    # {
    #   "metadashboard" : {
    #     "Community": {
    #       "Attention": "Community_Attention"
    #     },
    #     "Product": {
    #       "Vitality": "Product_Vitality"
    #     }
    #   }
    # }

    kibana_menu = {"metadashboard": {}}
    for entry in qm_menu.keys():
        [goal, attribute] = entry.split("_")
        if goal not in kibana_menu["metadashboard"]:
            kibana_menu["metadashboard"][goal] = {}
        kibana_menu["metadashboard"][goal][attribute] = qm_menu[entry]

    # Add the assessment dashboard
    kibana_menu["metadashboard"].update(assessment_menu)

    # First step is to fix the .kibana mapping to avoid dynamic mapping for the menu
    menu_mapping = """
    {
        "properties": {
            "metadashboard" : {
              "type": "object",
              "dynamic": "true"
            }
        }
    } """

    res = requests.put(es_url + "/.kibana/_mapping/doc", headers=ES_HEADERS, verify=False,
                       data=menu_mapping)
    res.raise_for_status()

    # Upload the menu created
    res = requests.put(es_url + "/.kibana/doc/metadashboard", headers=ES_HEADERS, verify=False,
                       data=json.dumps(kibana_menu))
    res.raise_for_status()
    logging.debug("Menu created: %s" % json.dumps(kibana_menu, indent=True))


def build_dashboards(es_url, kibana_url, es_index, template_file, template_assess_file,
                     model_name, backend_metrics_data):
    """
    Create all the dashboards needed to viz a Quality Model

    :param es_url: Elasticsearch URL
    :param kibana_url: Kibana URL
    :param es_index: index in elasticsearch with the metrics data
    :param template_file: template file to be used for building the attribute dashboards
    :param template_assess_file: template file to be used for the assessment dashboard
    :param model_name: quality model to use
    :param backend_metrics_data: backend to use to collect the metrics data
    :return:
    """

    logging.debug('Building the dashboards for the model: %s', model_name)

    if not template_assess_file:
        template_assess_file = ASSESS_PANEL

    qm_menu = {}  # Kibana menu for accessing the Quality Model dashboards
    assess_menu = {}  # Kibana menu for accessing the assessment dashboard

    # Check that the model and the template dashboard exists
    model_orm = None
    try:
        model_orm = QualityModel.objects.get(name=model_name)
    except QualityModel.DoesNotExist:
        logging.error('Can not find the metrics model %s', model_name)
        raise RuntimeError("Can not find the metrics model " + model_name)

    # Build a new dashboard for each attribute in the quality model
    for goal in model_orm.goals.all():
        for attribute in goal.attributes.all():
            dash_json = build_dashboard(es_url, kibana_url, es_index, template_file, goal,
                                        attribute, backend_metrics_data)
            qm_menu[dash_json['dashboard']['value']['title']] = dash_json['dashboard']['id']

    # Project assessment is included also in the viz
    assess(es_url, es_index, model_name, backend_metrics_data)
    # Upload the radar viz to show the assessment
    assess_dash = VizTemplatesData.read_template(template_assess_file)
    feed_dashboard(assess_dash, es_url, kibana_url)
    assess_menu[assess_dash['dashboard']['value']['title']] = assess_dash['dashboard']['id']

    build_menu(es_url, qm_menu, assess_menu)


if __name__ == '__main__':

    args = get_params()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
        logging.debug("Debug mode activated")
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')
    logging.getLogger("urllib3").setLevel(logging.WARNING)
    logging.getLogger("requests").setLevel(logging.WARNING)

    build_dashboards(args.elastic_url, args.kibana_url, args.index,
                     args.template_file, args.template_assess_file,
                     args.model, args.backend_metrics_data)



from prosoul.models import Attribute, QualityModel, Goal, Metric, MetricData


class AttributesData():

    def __init__(self, state):
        self.state = state

    def __find_attribute_subattributes(self, attribute):
        for subattr in attribute.subattributes.all():
            yield subattr
            for subsubattr in self.__find_attribute_subattributes(subattr):
                yield subsubattr

    def __find_goals_attributes(self, goals):
        for goal in goals:
            for attribute in goal.attributes.all():
                yield attribute
                for subattribute in self.__find_attribute_subattributes(attribute):
                    yield subattribute
            for attribute in self.__find_goals_attributes(goal.subgoals.all()):
                yield attribute

    def fetch(self):
        if not self.state or self.state.is_empty():
            attributes = Attribute.objects.all()
            for attribute in attributes:
                yield attribute
        elif self.state.attributes:
            attributes = Attribute.objects.filter(id__in=self.state.attributes)
            for attribute in attributes:
                yield attribute
        elif self.state.metrics:
            metrics = Metric.objects.filter(id__in=self.state.metrics)
            for attribute in self.__fetch_from_metrics(metrics):
                yield attribute
        elif self.state.goals:
            goals = Goal.objects.filter(id__in=self.state.goals)
            for attribute in self.__find_goals_attributes(goals):
                yield attribute
        elif self.state.qmodel_id:
            qmodel = QualityModel.objects.get(id=self.state.qmodel_id)
            goals = qmodel.goals.all()
            for attribute in self.__find_goals_attributes(goals):
                yield attribute


class QualityModelsData():

    def __init__(self, state=None):
        self.state = state

    def fetch(self):
        for qmodel in QualityModel.objects.all():
            yield qmodel


class GoalsData():

    def __init__(self, state):
        self.state = state

    def fetch(self):
        if not self.state or self.state.is_empty():
            for goal in Goal.objects.all():
                yield goal
        elif self.state.goals:
            goals = Goal.objects.filter(id__in=self.state.goals)
            for goal in goals:
                yield goal
        elif self.state.attributes:
            attributes = Attribute.objects.filter(id__in=self.state.attributes).values_list("id")
            goals = Goal.objects.filter(attributes__in=list(attributes))
            for goal in goals:
                yield goal
        elif self.state.metrics:
            metrics = Metric.objects.filter(id__in=self.state.metrics).values_list("id")
            attributes = Attribute.objects.filter(metrics__in=list(metrics)).values_list("id")
            goals = Goal.objects.filter(attributes__in=list(attributes))
            for goal in goals:
                yield goal
        elif self.state.qmodel_id:
            qmodel = QualityModel.objects.get(id=self.state.qmodel_id)
            goals = qmodel.goals.all()
            for goal in goals:
                yield goal
                for subgoal in goal.subgoals.all():
                    yield subgoal


class MetricsData():

    def __init__(self, state=None):
        self.state = state

    def __find_goals_metrics(self, goals):
        for goal in goals:
            for attribute in goal.attributes.all():
                for metric in attribute.metrics.all():
                    yield metric
            for metric in self.__find_goals_metrics(goal.subgoals.all()):
                yield metric

    def fetch(self):
        if not self.state or self.state.is_empty():
            for metric in Metric.objects.all():
                yield metric
        elif self.state.metrics:
            metrics = Metric.objects.filter(id__in=self.state.metrics)
            for metric in metrics:
                yield metric
        elif self.state.attributes:
            attributes = Attribute.objects.filter(id__in=self.state.attributes)
            for attribute in attributes:
                for metric in attribute.metrics.all():
                    yield metric
        elif self.state.goals:
            goals = Goal.objects.filter(id__in=self.state.goals)
            for metric in self.__find_goals_metrics(goals):
                yield metric
        elif self.state.qmodel_id:
            qmodel = QualityModel.objects.get(id=self.state.qmodel_id)
            for metric in self.__find_goals_metrics(qmodel.goals.all()):
                yield metric


class MetricsDataData():

    def __init__(self, state=None):
        self.state = state

    def fetch(self):
        for metric_data in MetricData.objects.all():
            yield metric_data



# Supported backends for providing the data for metrics in the quality models
BACKEND_METRICS_DATA = ['grimoirelab', 'ossmeter']


def find_metric_name_field(backend_metrics_data):
    """ Find the field with the metric name for a given backend for metrics """

    if backend_metrics_data not in BACKEND_METRICS_DATA:
        raise RuntimeError('Backend for metrics data not supported: ' + backend_metrics_data)

    if backend_metrics_data == 'grimoirelab':
        metric_name = 'metadata__gelk_backend_name'
    elif backend_metrics_data == 'ossmeter':
        metric_name = 'metric_es_name.keyword'

    return metric_name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Import Metrics Models in Prosoul
#
# Copyright (C) 2017 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Authors:
#   Alvaro del Castillo San Felix <acs@bitergia.com>
#

import argparse
import json
import logging
import os

from time import time

import django
# settings.configure()
os.environ['DJANGO_SETTINGS_MODULE'] = 'django_prosoul.settings'
django.setup()

from django.test import TestCase

from prosoul.models import Attribute, DataSourceType, Factoid, Goal, Metric, MetricData, QualityModel

from prosoul.prosoul_export import fetch_models, gl2alambic, gl2ossmeter, show_report

SUPPORTED_FORMATS = ['alambic', 'grimoirelab', 'ossmeter']


def get_params():
    parser = argparse.ArgumentParser(usage="usage: prosoul_import.py [options]",
                                     description="Import Metrics Models in Prosoul")
    parser.add_argument('-g', '--debug', action='store_true')
    parser.add_argument("-f", "--file", required=True,
                        help="File path from which to load the Metrics Models")
    parser.add_argument('--format', default='grimoirelab',
                        help="Import file format (default grimoirelab)")
    parser.add_argument('-c', '--check', action='store_true',
                        help='Export the data and compare it with the imported')

    return parser.parse_args()


def add(cls_orm, **params):
    """ Add an object if it does not exists """

    obj_orm = None

    try:
        obj_orm = cls_orm.objects.get(**params)
        # logging.debug('Already exists %s: %s', cls_orm.__name__, params)
    except cls_orm.DoesNotExist:
        obj_orm = cls_orm(**params)
        try:
            obj_orm.save()
            logging.debug('Added %s: %s', cls_orm.__name__, params)
        except django.db.utils.IntegrityError as ex:
            obj_orm = None
            logging.error("Can't add %s: %s", cls_orm.__name__, params)
            logging.error(ex)

    return obj_orm


def feed_models(models_json):

    def feed_attribute(attribute):
        aparams = {"name": attribute['name']}
        if 'description' in attribute:
            aparams["description"] = attribute['description']
        attribute_orm = add(Attribute, **aparams)

        if 'subattributes' in attribute:
            for subattribute in attribute['subattributes']:
                subattribute_orm = feed_attribute(subattribute)
                attribute_orm.subattributes.add(subattribute_orm)

        for metric in attribute['metrics']:
            data_source_orm = None
            if 'data_source_type' in metric and metric['data_source_type']:
                dsparams = {"name": metric['data_source_type']}
                data_source_orm = add(DataSourceType, **dsparams)
            metric_data_orm = None
            if 'data_implementation' in metric and metric['data_implementation']:
                mdparams = {"implementation": metric['data_implementation'],
                            "params": metric['data_params']}
                metric_data_orm = add(MetricData, **mdparams)

            metparams = {"name": metric['name'],
                         "data_source_type": data_source_orm,
                         "data": metric_data_orm}
            if 'thresholds' in metric and metric['thresholds']:
                metparams['thresholds'] = metric['thresholds']
            metric_orm = add(Metric, **metparams)
            if metric_orm is None:
                raise RuntimeError("Can not import attribute " + attribute['name'])
            attribute_orm.metrics.add(metric_orm)

        if 'factoids' in attribute:
            for factoid in attribute['factoids']:
                data_source_orm = None
                if 'data_source_type' in factoid and factoid['data_source_type']:
                    dsparams = {"name": factoid['data_source_type']}
                    data_source_orm = add(DataSourceType, **dsparams)
                fparams = {"name": factoid['name'],
                           "data_source_type": data_source_orm}
                factoid_orm = add(Factoid, **fparams)
                attribute_orm.factoids.add(factoid_orm)

        attribute_orm.save()

        return attribute_orm

    def feed_goal(goal):
        gparams = {"name": goal['name']}
        goal_orm = add(Goal, **gparams)

        if 'subgoals' in goal:
            for subgoal in goal['subgoals']:
                subgoal_orm = feed_goal(subgoal)
                goal_orm.subgoals.add(subgoal_orm)

        for attribute in goal['attributes']:
            attribute_orm = feed_attribute(attribute)
            goal_orm.attributes.add(attribute_orm)

        goal_orm.save()

        return goal_orm

    for model in models_json['qualityModels']:
        mparams = {"name": model['name']}
        if 'version' in model:
            mparams["version"] = model['version']
        model_orm = add(QualityModel, **mparams)

        for goal in model['goals']:
            goal_orm = feed_goal(goal)
            model_orm.goals.add(goal_orm)

        model_orm.save()


def alambic2gl(model_json):
    """ Convert a JSON from Alambic format to GrimoireLab """

    def build_attribute(attribute):
        """ The same attribute concept than in GrimoireLab """
        attribute_json = {"name": attribute['mnemo'], "subattributes": [],
                          "metrics": [], "factoids": []}
        for child in attribute['children']:
            if child['type'] in ['attribute', 'concept']:
                attribute_json['subattributes'].append(build_attribute(child))
            elif child['type'] == 'metric':
                metric_json = {"name": child['mnemo']}
                attribute_json['metrics'].append(metric_json)
            else:
                raise RuntimeError('Type of attribute not supported ' + child['type'])

        return attribute_json

    def build_goal(child):
        """ The first level attribute is a Goal in GrimoireLab """
        goal_json = {"name": child['mnemo'], "attributes": [],
                     "subgoals": []}

        for attribute in child['children']:
            # All all attributes with subattributes until metrics are reached
            attribute_json = build_attribute(attribute)
            goal_json["attributes"].append(attribute_json)

        return goal_json

    logging.debug('Converting from Alambic to GrimoireLab quality model')

    grimoirelab_json = {"qualityModels": []}

    gl_model_json = {"name": model_json["name"], "goals": []}

    for child in model_json['children']:
        # In alambic all are attributes
        # Convert the first level attribute to a goal
        goal_json = build_goal(child)

        gl_model_json["goals"].append(goal_json)

    grimoirelab_json["qualityModels"].append(gl_model_json)

    return grimoirelab_json


def ossmeter2gl(model_json):
    """ Convert a JSON from OSSMeter format to GrimoireLab """

    def build_goal(qualityAspect):
        # A qualityAspect is a Goal in GrimoireLab
        goal_json = {"name": qualityAspect['name'], "attributes": [],
                     "subgoals": []}

        for attribute_om in qualityAspect['attributes']:
            attribute_json = {"name": attribute_om['name'],
                              "description": attribute_om['description'],
                              "metrics": [], "factoids": []}
            for metric_om in attribute_om['metrics']:
                metrics_json = {"name": metric_om}
                attribute_json['metrics'].append(metrics_json)
            for factoid_om in attribute_om['factoids']:
                factoid_json = {"name": factoid_om}
                attribute_json['factoids'].append(factoid_json)

            goal_json["attributes"].append(attribute_json)

        if 'qualityAspects' in qualityAspect:
            for subqualityAspect in qualityAspect['qualityAspects']:
                subgoal_json = build_goal(subqualityAspect)
                goal_json["subgoals"].append(subgoal_json)

        return goal_json

    logging.debug('Converting from OSSMeter to GrimoireLab quality model')

    grimoirelab_json = {"qualityModels": []}

    model_json = model_json['qualityModel']

    gl_model_json = {"name": model_json["name"], "goals": []}

    for qualityAspect in model_json['qualityAspects']:
        goal_json = build_goal(qualityAspect)

        gl_model_json["goals"].append(goal_json)

    grimoirelab_json["qualityModels"].append(gl_model_json)

    return grimoirelab_json


def convert_to_grimoirelab(format_, model_json):
    """ Convert a json from supported format_ to grimoirelab format """

    grimoirelab_json = {}

    if format_ not in SUPPORTED_FORMATS:
        raise RuntimeError("Quality Model format not supported %s" % format_)

    if format_ == 'grimoirelab':
        return model_json
    elif format_ == 'ossmeter':
        grimoirelab_json = ossmeter2gl(model_json)
    elif format_ == 'alambic':
        grimoirelab_json = alambic2gl(model_json)

    return grimoirelab_json


def compare_models(models_json, format_=None):
    # The Prosoul database must only contains the models imported
    exported_models_json = fetch_models()
    if format_ == 'ossmeter':
        exported_models_json = gl2ossmeter(exported_models_json)
    elif format_ == 'alambic':
        exported_models_json = gl2alambic(exported_models_json)
    test = TestCase()
    test.maxDiff = None
    test.assertDictEqual(models_json, exported_models_json)
    logging.info("Check completed")


if __name__ == '__main__':
    task_init = time()

    args = get_params()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
        logging.debug("Debug mode activated")
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')
    logging.getLogger("urllib3").setLevel(logging.WARNING)
    logging.getLogger("requests").setLevel(logging.WARNING)

    logging.info("Importing models from file %s", args.file)
    with open(args.file) as fmodel:
        import_models_json = json.load(fmodel)
        models_json = import_models_json
        if args.format != "grimoirelab":
            models_json = convert_to_grimoirelab(args.format, import_models_json)
        feed_models(models_json)

        show_report(models_json)

        logging.debug("Total importing time ... %.2f sec", time() - task_init)

        if args.check:
            logging.info('Checking data ...')
            compare_models(import_models_json, args.format)



import json
import logging
import os.path

import pkg_resources

PANELS_DIR = 'panels'


class VizTemplatesData():

    def fetch(self):
        """
        Read available panel templates and return them

        :return: dict with the path to the template as key  and its content as value
        """

        # The template files could be located inside the pip package
        templates = pkg_resources.resource_listdir(__name__, PANELS_DIR)

        for tfilename in templates:
            resource_path = '/'.join((PANELS_DIR, tfilename))
            if pkg_resources.resource_isdir(__name__, resource_path):
                continue
            tstring = pkg_resources.resource_string(__name__, resource_path).decode("utf-8")
            template = json.loads(tstring)['dashboard']['value']['title']
            yield (resource_path, template)

    @classmethod
    def read_template(self, tpath):
        """
        Read the contents of a template file trying to read it from the file system
        and if fails, as a resource inside a pip package.

        :param tpath: path to the template file/resource
        :return: a dict with the contents of the template
        """
        if os.path.isfile(tpath):
            template_file = open(tpath, "r")
            dashboard = json.load(template_file)
            template_file.close()
        else:
            logging.debug("Template file not found. Trying to load it from the pip package.")
            tstring = pkg_resources.resource_string(__name__, tpath).decode("utf-8")
            logging.debug("Template loaded from the pip package")
            dashboard = json.loads(tstring)

        return dashboard



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Load CROSSMINER metrics definition in Prosoul
#
# Copyright (C) 2017 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Authors:
#   Alvaro del Castillo San Felix <acs@bitergia.com>
#

import argparse
import logging
import os


from time import time

import requests

import django
# settings.configure()
os.environ['DJANGO_SETTINGS_MODULE'] = 'django_prosoul.settings'
django.setup()

from prosoul.models import DataSourceType, Metric


def get_params():
    parser = argparse.ArgumentParser(usage="usage: metrics_import.py [options]",
                                     description="Load CROSSMINER metrics definition in Prosoul")
    parser.add_argument("-e", "--elastic-url", required=True,
                        help="Elasticsearch URL with the metrics")
    parser.add_argument('-g', '--debug', action='store_true')
    parser.add_argument('-i', '--index', required='True', help='Index with the metrics')

    return parser.parse_args()


def search_agg():
    query = """
    {
      "query": {
        "bool": {
          "must": []
        }
      },
      "size": 0,
      "aggs": {
        "2": {
          "terms": {
            "field": "metric_class.keyword",
            "size": 10000,
            "order": {
              "_count": "desc"
            }
          },
          "aggs": {
            "3": {
              "terms": {
                "field": "metric_name.keyword",
                "size": 10000,
                "order": {
                  "_count": "desc"
                }
              },
              "aggs": {
                "4": {
                  "terms": {
                    "field": "metric_es_name.keyword",
                    "size": 10000,
                    "order": {
                      "_count": "desc"
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    """

    return query


def process_agg(res):
    metrics = {}

    for data_source in res.json()["aggregations"]["2"]["buckets"]:
        if data_source['key'] not in metrics:
            metrics[data_source['key']] = {}

        for metric_class in data_source["3"]["buckets"]:
            if metric_class['key'] not in metrics[data_source['key']]:
                metrics[data_source['key']][metric_class['key']] = []

            for metric_name in metric_class["4"]["buckets"]:
                if len(metric_name['key'].split(":")) > 4:
                    # Bad metrics: 12:18:43:11:000_bugsConsidered
                    continue
                metrics[data_source['key']][metric_class['key']].append(metric_name['key'])

    return metrics


def add(cls_orm, **params):
    """ Add an object if it does not exists """

    obj_orm = None

    try:
        obj_orm = cls_orm.objects.get(**params)
        # logging.debug('Already exists %s: %s', cls_orm.__name__, params)
    except cls_orm.DoesNotExist:
        obj_orm = cls_orm(**params)
        try:
            obj_orm.save()
            logging.debug('Added %s: %s', cls_orm.__name__, params)
        except django.db.utils.IntegrityError as ex:
            logging.error("Can't add %s: %s", cls_orm.__name__, params)
            logging.error(ex)

    return obj_orm


def load_metrics(es_url, index):

    search_url = es_url + "/" + index + "/_search"

    res = requests.post(search_url, data=search_agg())

    metrics = process_agg(res)

    ndata_sources = 0
    nmetrics = 0

    for data_source in metrics:
        pparams = {"name": data_source}
        data_source_orm = add(DataSourceType, **pparams)

        ndata_sources += 1

        for metric_class in metrics[data_source]:
            metric_params = {"mclass": metric_class, "data_source_type": data_source_orm}
            for metric in metrics[data_source][metric_class]:
                metric_params["name"] = metric
                add(Metric, **metric_params)

                nmetrics += 1

        data_source_orm.save()

    return nmetrics


if __name__ == '__main__':

    task_init = time()

    args = get_params()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
        logging.debug("Debug mode activated")
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')
    logging.getLogger("urllib3").setLevel(logging.WARNING)
    logging.getLogger("requests").setLevel(logging.WARNING)

    nmetrics = load_metrics(args.elastic_url, args.index)

    logging.debug("Total loading time ... %.2f sec", time() - task_init)
    print("Metrics loaded", nmetrics)



import json

from django import shortcuts
from django.http import HttpResponse
from django.template import loader

from prosoul.prosoul_export import fetch_models, gl2viewer
from prosoul.prosoul_assess import assess
from prosoul.prosoul_vis import build_dashboards
from prosoul.views_editor import editor
from prosoul.forms import AssessmentForm, VisualizationForm


class Viewer():
    @staticmethod
    def viewer(request):
        """ Basic Models Viewer just dumping the JSON of all models """
        models = fetch_models()
        if not models['qualityModels']:
            return editor(request)
        model_selected = models['qualityModels'][0]['name']
        if request.method == 'GET' and 'qmodel_selected' in request.GET:
            model_selected = request.GET['qmodel_selected']
        viewer_data = gl2viewer(models, model_name=model_selected)
        context = {'active_page': "viewer",
                   'qmodel_selected': model_selected,
                   'qmodels': models['qualityModels'],
                   'qm_data': viewer_data[0],
                   'qm_data_str': json.dumps(viewer_data[0]).replace('\"', '\\"'),
                   'attributes_data': viewer_data[1],
                   'attributes_data_str': json.dumps(viewer_data[1]).replace('\"', '\\"'),
                   'metrics_data': viewer_data[2],
                   'metrics_data_str': json.dumps(viewer_data[2]).replace('\"', '\\"')}

        template = loader.get_template('prosoul/viewer.html')

        render_index = template.render(context, request)

        return HttpResponse(render_index)


class Visualize():
    @staticmethod
    def visualize(request):
        template = loader.get_template('prosoul/visualize.html')
        context = {'active_page': "visualize", "vis_config_form": VisualizationForm()}
        render_index = template.render(context, request)
        return HttpResponse(render_index)

    @staticmethod
    def create(request):
        error = None
        if request.method == 'POST':
            form = VisualizationForm(request.POST)
            context = {'active_page': "visualize", "vis_config_form": form}
            if form.is_valid():
                qmodel_name = form.cleaned_data['quality_model']
                es_url = form.cleaned_data['es_url']
                kibana_url = form.cleaned_data['kibana_url']
                es_index = form.cleaned_data['es_index']
                attribute_template = form.cleaned_data['attribute_template']
                backend_metrics_data = form.cleaned_data['backend_metrics_data']

                # Time to execute the visualization creation
                try:
                    assess_template = None
                    build_dashboards(es_url, kibana_url, es_index, attribute_template, assess_template,
                                     qmodel_name, backend_metrics_data)
                except Exception as ex:
                    error = "Problem creating the visualizations " + str(ex)

                context.update({"errors": error})
                if not error:
                    context.update({"kibana_url": kibana_url})
                return shortcuts.render(request, 'prosoul/visualize.html', context)
            else:
                context.update({"errors": form.errors})
                return shortcuts.render(request, 'prosoul/visualize.html', context)
        else:
            return shortcuts.render(request, 'prosoul/visualize.html', {"error": "Use GET method to send data"})


class Assessment():

    @staticmethod
    def assess(request):
        template = loader.get_template('prosoul/assessment.html')
        context = {'active_page': "assess", "assess_config_form": AssessmentForm()}
        render_index = template.render(context, request)
        return HttpResponse(render_index)

    @staticmethod
    def render_attribute_table(metrics, goal):
        table = "<table class='table'>"
        # Headers
        table += "<thead><th scope='col'>Attribute</th>"
        for metric in metrics:
            # Clean the metric name
            # "GitHubEnrich {\"filter\": {\"term\": {\"state\": \"closed\"}}}
            if len(metric.split()) == 1:
                table += "<th>%s</th>" % metric
            else:
                table += "<th>%s + filter</th>" % metric.split()[0]
        table += "</thead>"
        for attribute in goal:
            # One row per atribute with its metrics
            table += "<tr><th scope='row'>%s</td>" % attribute
            # Let's find the metrics to fill the metrics columns
            for metric_col in metrics:
                metric_col_found = False
                for metric in goal[attribute]:
                    if metric == metric_col:
                        table += "<td>%s</td>" % goal[attribute][metric]
                        metric_col_found = True
                        break
                if not metric_col_found:
                    table += "<td></td>"
            table += "</tr>"
        table += "</table>"

        return table

    def render_tables(assessment):
        """ Convert the JSON with the assessmet in an HTML table

        Sample format:

        {
         "Community": {
          "Attention": {
           "GitHubEnrich {\"filter\": {\"term\": {\"state\": \"closed\"}}}": {
            "perceval": 5,
            "GrimoireELK": 5
           }
          }
         },
         "Product": {
          "Vitality": {
           "GitHubEnrich": {
            "perceval": 2,
            "GrimoireELK": 2
           },
           "GitEnrich": {
            "perceval": 3,
            "GrimoireELK": 3
           }
          }
         }
        }
        """

        projects_data = {}
        metrics = []

        for goal in assessment:
            for attribute in assessment[goal]:
                for metric in assessment[goal][attribute]:
                    metrics.append(metric)
                    for project in assessment[goal][attribute][metric]:
                        if project not in projects_data:
                            projects_data[project] = {}
                        if goal not in projects_data[project]:
                            projects_data[project][goal] = {}
                        if attribute not in projects_data[project][goal]:
                            projects_data[project][goal][attribute] = {}
                        projects_data[project][goal][attribute][metric] = assessment[goal][attribute][metric][project]

        print(projects_data)

        metrics = list(set(metrics))
        # TODO: move this table rendering to Django templates
        tables = ""
        for project in projects_data:
            tables += "<h3>Project: " + project + "</h3>"
            for goal in projects_data[project]:
                tables += "<h5>Goal: " + goal + "</h5>"
                tables += Assessment.render_attribute_table(metrics, projects_data[project][goal])

        return tables

    @staticmethod
    def create(request):
        error = None
        if request.method == 'POST':
            form = AssessmentForm(request.POST)
            context = {'active_page': "assess", "assess_config_form": form}
            if form.is_valid():
                qmodel_name = form.cleaned_data['quality_model']
                es_url = form.cleaned_data['es_url']
                es_index = form.cleaned_data['es_index']
                backend_metrics_data = form.cleaned_data['backend_metrics_data']

                # Time to execute the assessment creation
                try:
                    assessment = assess(es_url, es_index, qmodel_name, backend_metrics_data)
                except Exception as ex:
                    error = "Problem creating the assessment " + str(ex)

                context.update({"errors": error})
                if not error:
                    assessment_table = Assessment.render_tables(assessment)
                    if assessment_table:
                        context.update({"assessment": Assessment.render_tables(assessment)})
                    else:
                        context.update({"errors": "Empty assessment. Review the form data."})
                return shortcuts.render(request, 'prosoul/assessment.html', context)
            else:
                context.update({"errors": form.errors})
                return shortcuts.render(request, 'prosoul/assessment.html', context)
        else:
            return shortcuts.render(request, 'prosoul/assessment.html', {"error": "Use POST method to send data"})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# ProSoul Metrics Manager Tool
#
# Copyright (C) Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Authors:
#   Alvaro del Castillo San Felix <acs@bitergia.com>
#

import argparse
import logging
import os
from statistics import mean, median, stdev

import matplotlib.pyplot as plot

import django
# settings.configure()
os.environ['DJANGO_SETTINGS_MODULE'] = 'django_prosoul.settings'
django.setup()

from prosoul.models import QualityModel
from prosoul.prosoul_assess import compute_metric_per_project


def get_params():
    parser = argparse.ArgumentParser(usage="usage: mdashboard.py [options]",
                                     description="Create a Kibana Dashboard to show a Quality Model")
    parser.add_argument("-e", "--elastic-url", required=True,
                        help="Elasticsearch URL with the metrics")
    parser.add_argument('-g', '--debug', action='store_true')
    parser.add_argument('-i', '--index', required=True, help='Index with the metrics')
    parser.add_argument('-m', '--model', required=True,
                        help='Model to be used to build the Dashboard')
    parser.add_argument('-b', '--backend-metrics-data', default='grimoirelab',
                        help='Backend metrics data to use (grimoirelab, ossmeter, ...)')
    parser.add_argument('-l', '--list', action='store_true', help='List available metrics')
    parser.add_argument('--metrics', nargs='*', help='Metrics names to be computed')
    parser.add_argument('--plot', action='store_true',
                        help='Show a plot with the metrics values (use it with --metric option)')

    return parser.parse_args()


def compute_metrics(names, es_url, es_index, model_name, backend_metrics_data):
    """
    Compute the value for a list of metrics
    :param names: list with the names of the metrics to be computed
    :return: a dict with the metric name as key and the metric values as value
    """

    metrics_values = {}

    for name in names:
        metrics_values[name] = compute_metric(name, es_url, es_index, model_name, backend_metrics_data)

    return metrics_values


def compute_metric(name, es_url, es_index, model_name, backend_metrics_data):
    """
    Compute the value for a metric

    :param name: name of the metric to be computed
    :param es_url: Elasticsearch URL
    :param es_index: index with the metrics data
    :param model_name: quality model name to be used
    :param backend_metrics_data: backend used to collect the metrics data
    :return: a list with the metric value per project
    """

    metrics = list_metrics(es_url, es_index, model_name, backend_metrics_data)
    metrics_name = [str(metric) for metric in metrics]

    # Check that the metric exists
    if name not in metrics_name:
        raise RuntimeError("metric %s not found (available %s)" % (name, metrics_name))

    # Check that the metric has data
    metric = None
    for m in metrics:
        if str(m) == name:
            metric = m
            break

    if not metric.data:
        raise RuntimeError("metric %s has no data" % name)

    # Time to compute the metric
    metric_value = compute_metric_per_project(es_url, es_index, metric.data, backend_metrics_data)

    return metric_value


def list_metrics(es_url, es_index, model_name, backend_metrics_data):
    """
    List available metrics

    :param es_url: Elasticsearch URL
    :param es_index: index with the metrics data
    :param model_name: quality model name to be used
    :param backend_metrics_data: backend used to collect the metrics data
    :return: a list with the metrics available

    """

    logging.debug("Listing the metrics available")

    metrics = []

    # Check that the model exists
    model_orm = None
    try:
        model_orm = QualityModel.objects.get(name=model_name)
    except QualityModel.DoesNotExist:
        RuntimeError('Can not find the metrics model %s' + model_name)

    for goal in model_orm.goals.all():
        for attribute in goal.attributes.all():
            for metric in attribute.metrics.all():
                metrics.append(metric)

    return metrics


def show_metric_stats(name, metric_project_values, plot_data=False):
    """
    Show a report in standard output about the values of a metric in all projects

    :param name: name of the metric to be plotted
    :param metric_value: dict with the value of the metric per each project
    :param plot_data: show a plot with the data
    :return:
    """

    metrics = []

    print("Total number of projects for %s: %i" % (name, len(metric_project_values)))
    for project in metric_project_values:
        metrics.append(project['metric'])

    if not metrics:
        return

    # Show the max, min, median, mean and standard deviation of the metric
    print("Max:", max(metrics), "Min:", min(metrics), "Mean:", round(mean(metrics), 2),
          "Median:", round(median(metrics), 2), "Stdev:", round(stdev(metrics), 2))

    if plot_data:
        # Plot the metrics
        x = range(0, len(metrics))
        plot.plot(x, metrics)
        plot.title(name)
        plot.show()


if __name__ == '__main__':

    args = get_params()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
        logging.debug("Debug mode activated")
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')
    logging.getLogger("urllib3").setLevel(logging.WARNING)
    logging.getLogger("requests").setLevel(logging.WARNING)

    if args.list:
        metrics = list_metrics(args.elastic_url, args.index, args.model, args.backend_metrics_data)
        print(metrics)
    elif args.metrics:
        metrics_value = compute_metrics(args.metrics, args.elastic_url, args.index, args.model, args.backend_metrics_data)
        for name, metric_value in metrics_value.items():
            show_metric_stats(name, metric_value, plot_data=args.plot)



from django.db import models
from django.contrib.auth.models import User
from django.core.validators import validate_comma_separated_integer_list


class ProsoulModel(models.Model):
    """ Basic metadata for Prosoul objects """
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    active = models.BooleanField(default=True)
    description = models.CharField(max_length=1024, default='', null=True, blank=True)

    created_by = models.ForeignKey(User, null=True, blank=True, on_delete=models.SET_NULL)

    class Meta:
        abstract = True


class DataSourceType(ProsoulModel):
    name = models.CharField(max_length=200, unique=True)

    def __str__(self):
        return self.name


class MetricData(ProsoulModel):
    # Name of the metric in Elasticsearch with the data for the metric
    implementation = models.CharField(max_length=200, null=True, blank=True)
    # params to be used to compute the metric, including filters
    params = models.CharField(max_length=1024, null=True, blank=True)

    def __str__(self):
        if self.params:
            return self.implementation + " " + self.params
        return self.implementation


class Metric(ProsoulModel):
    name = models.CharField(max_length=200, unique=True)
    data = models.ForeignKey(MetricData, on_delete=models.CASCADE, null=True, blank=True)

    data_source_type = models.ForeignKey(DataSourceType,
                                         on_delete=models.CASCADE, null=True, blank=True)

    thresholds = models.CharField(max_length=200, default=None, null=True, blank=True,
                                  validators=[validate_comma_separated_integer_list])

    def __str__(self):
        return self.name


class Factoid(ProsoulModel):
    name = models.CharField(max_length=200, unique=True)

    data_source_type = models.ForeignKey(DataSourceType,
                                         on_delete=models.CASCADE, null=True, blank=True)

    def __str__(self):
        return self.name


class Attribute(ProsoulModel):
    name = models.CharField(max_length=200, unique=False)
    # Relations
    metrics = models.ManyToManyField(Metric, blank=True)
    factoids = models.ManyToManyField(Factoid, blank=True)
    subattributes = models.ManyToManyField("Attribute", blank=True)

    def __str__(self):
        return self.name


class Goal(ProsoulModel):
    name = models.CharField(max_length=200, unique=False)
    # Relations
    attributes = models.ManyToManyField(Attribute)
    subgoals = models.ManyToManyField("Goal", blank=True)

    def __str__(self):
        return self.name


class QualityModel(ProsoulModel):
    """ Quality Model (maturity, Health ...)"""
    name = models.CharField(max_length=200, unique=True)
    # Relations
    goals = models.ManyToManyField("Goal")

    def __str__(self):
        return self.name



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-03-05 09:39
from __future__ import unicode_literals

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import re


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Attribute',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, default='', max_length=1024, null=True)),
                ('name', models.CharField(max_length=200, unique=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DataSourceType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, default='', max_length=1024, null=True)),
                ('name', models.CharField(max_length=200, unique=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Factoid',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, default='', max_length=1024, null=True)),
                ('name', models.CharField(max_length=200, unique=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
                ('data_source_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='prosoul.DataSourceType')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Goal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, default='', max_length=1024, null=True)),
                ('name', models.CharField(max_length=200, unique=True)),
                ('attributes', models.ManyToManyField(to='prosoul.Attribute')),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
                ('subgoals', models.ManyToManyField(blank=True, to='prosoul.Goal')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Metric',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, default='', max_length=1024, null=True)),
                ('name', models.CharField(max_length=200, unique=True)),
                ('thresholds', models.CharField(blank=True, default=None, max_length=200, null=True, validators=[django.core.validators.RegexValidator(re.compile('^\\d+(?:\\,\\d+)*\\Z', 32), code='invalid', message='Enter only digits separated by commas.')])),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MetricData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, default='', max_length=1024, null=True)),
                ('implementation', models.CharField(blank=True, max_length=200, null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='QualityModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('description', models.CharField(blank=True, default='', max_length=1024, null=True)),
                ('name', models.CharField(max_length=200, unique=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
                ('goals', models.ManyToManyField(to='prosoul.Goal')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='metric',
            name='data',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='prosoul.MetricData'),
        ),
        migrations.AddField(
            model_name='metric',
            name='data_source_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='prosoul.DataSourceType'),
        ),
        migrations.AddField(
            model_name='attribute',
            name='factoids',
            field=models.ManyToManyField(blank=True, to='prosoul.Factoid'),
        ),
        migrations.AddField(
            model_name='attribute',
            name='metrics',
            field=models.ManyToManyField(blank=True, to='prosoul.Metric'),
        ),
        migrations.AddField(
            model_name='attribute',
            name='subattributes',
            field=models.ManyToManyField(blank=True, to='prosoul.Attribute'),
        ),
    ]



from django.contrib import admin
from . import models

# Register your models here.
admin.site.register(models.Attribute)
admin.site.register(models.DataSourceType)
admin.site.register(models.Factoid)
admin.site.register(models.Goal)
admin.site.register(models.Metric)
admin.site.register(models.MetricData)
admin.site.register(models.QualityModel)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2018 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>
#

import codecs
import os
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))
readme_md = os.path.join(here, 'README.md')

# Pypi wants the description to be in reStrcuturedText, but
# we have it in Markdown. So, let's convert formats.
# Set up thinkgs so that if pypandoc is not installed, it
# just issues a warning.
try:
    import pypandoc
    README = pypandoc.convert(readme_md, 'rst')
except (IOError, ImportError):
    print("Warning: pypandoc module not found, or pandoc not installed. "
          "Using md instead of rst")
    with codecs.open(readme_md, encoding='utf-8') as f:
        README = f.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-prosoul',
    version='0.2.0',
    packages=['prosoul'],
    include_package_data=True,
    license='GPLv3',
    description='Prosoul is a software quality models manager to create, import/export, view and edit models',
    long_description=README,
    url='https://github.com/bitergia/prosoul',
    author='Bitergia',
    author_email='acs@bitergia.com',
    keywords="development repositories analytics quality models assessment visualization",
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Customer Service',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'django>=2.0', 'matplotlib', 'grimoire-elk', 'sortinghat', 'kidash'
    ],
    python_requires='>=3.4'

)



import json

from django.test import TestCase

from .prosoul_import import compare_models, convert_to_grimoirelab, feed_models


class ProsoulImportExport(TestCase):

    def test_import_export_grimoirelab(self):
        """ Test the import/export process for quality models in grimoirelab format """
        format_ = 'grimoirelab'
        filepath = 'prosoul/data/grimoirelab_ossmeter_qm.json'
        fmodel = open(filepath, "r")
        import_models_json = json.load(fmodel)
        fmodel.close()
        if format_ != "grimoirelab":
            import_models_json = convert_to_grimoirelab(format_, import_models_json)
        feed_models(import_models_json)
        compare_models(import_models_json, format_)

    def off_test_import_export_alambic(self):
        """ Test the import/export process for quality models in alambic format """
        # TODO is failing now because assertDictEqual and keys ordering
        format_ = 'alambic'
        filepath = 'prosoul/data/alambic_quality_model.json'
        fmodel = open(filepath, "r")
        import_models_json = json.load(fmodel)
        fmodel.close()
        models = import_models_json
        if format_ != "grimoirelab":
            models = convert_to_grimoirelab(format_, import_models_json)
        feed_models(models)
        compare_models(import_models_json, format_)

    def off_test_import_export_ossmeter(self):
        """ Test the import/export process for quality models in ossmeter format """
        # TODO is failing now because assertDictEqual and keys ordering
        format_ = 'ossmeter'
        filepath = 'prosoul/data/ossmeter_qm.json'
        fmodel = open(filepath, "r")
        import_models_json = json.load(fmodel)
        fmodel.close()
        models = import_models_json
        if format_ != "grimoirelab":
            models = convert_to_grimoirelab(format_, import_models_json)
        feed_models(models)
        compare_models(import_models_json, format_)
